"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var _find = require("lodash/find");
var inversify_1 = require("inversify");
var H = require("./di_interfaces");
var block_provider_1 = require("./block_provider");
var di_symbols_1 = require("./di_symbols");
var ActionStateStoreImpl = (function () {
    function ActionStateStoreImpl(block_registry, event_bus, u) {
        this.block_registry = block_registry;
        this.event_bus = event_bus;
        this.u = u;
        this.states = [];
    }
    ActionStateStoreImpl.prototype.clear = function () {
        this.states = [];
    };
    ActionStateStoreImpl.prototype.perform_automatic_state_transitions = function () {
        for (var i = 0; i < this.states.length; i++) {
            var s = this.states[i];
            if (s.spec.statefulness.automatic_transitions === 'one_frame') {
                s.value = '';
                continue;
            }
            var transition = s.spec.statefulness.automatic_transitions[s.value];
            if (transition != undefined) {
                s.value = transition;
            }
        }
    };
    ActionStateStoreImpl.prototype.update = function (new_actions) {
        this.perform_automatic_state_transitions();
        for (var i = 0; i < new_actions.length; i++) {
            var a = new_actions[i];
            if (a.value == undefined) {
                continue;
            }
            var action_id = block_provider_1.namespaced_id(a.namespace, a.id);
            var spec = this.block_registry.get_action_spec(action_id);
            if (spec == undefined) {
                this.event_bus.error.runtime.send({
                    error: this.u.ohno.system.action_received_without_spec({ action: a }),
                });
                continue;
            }
            if (spec.statefulness == undefined) {
                continue;
            }
            var state_params = {
                sub_type: a.sub_type,
            };
            if (spec.entity_specific) {
                state_params.entity_id = a.entity_id;
            }
            var spec_params = {
                id: action_id,
            };
            var matching_state = _find(this.states, tslib_1.__assign({}, state_params, { spec: spec_params }));
            if (matching_state != undefined) {
                matching_state.value = a.value;
            }
            else {
                this.states.push({
                    namespace: a.namespace,
                    entity_id: a.entity_id,
                    sub_type: a.sub_type,
                    value: a.value,
                    spec: spec,
                });
            }
        }
    };
    ActionStateStoreImpl.prototype.get_action_state_value = function (params) {
        var state_params = {
            namespace: params.action_namespace,
        };
        var spec_params = {
            id: params.action_id,
        };
        if (params.entity_id != undefined) {
            state_params.entity_id = params.entity_id;
        }
        if (params.sub_type != undefined) {
            state_params.sub_type = params.sub_type;
        }
        var matching = _find(this.states, tslib_1.__assign({}, state_params, { spec: spec_params }));
        if (matching != undefined) {
            return matching.value;
        }
        var id = block_provider_1.namespaced_id(params.action_namespace, params.action_id);
        var spec = this.block_registry.get_action_spec(id);
        if (spec == undefined) {
            throw this.u.ohno.system.state_query_received_without_spec({
                query_params: params,
            });
        }
        if (spec.statefulness == undefined) {
            throw this.u.ohno.system.state_query_for_non_stateful_action({
                query_params: params,
            });
        }
        return spec.statefulness.default_value;
    };
    ActionStateStoreImpl = tslib_1.__decorate([
        inversify_1.injectable(),
        tslib_1.__param(0, inversify_1.inject(di_symbols_1.BINDING.BlockRegistry)),
        tslib_1.__param(1, inversify_1.inject(di_symbols_1.BINDING.EventBus)),
        tslib_1.__param(2, inversify_1.inject(di_symbols_1.BINDING.Util)),
        tslib_1.__metadata("design:paramtypes", [Object, Object, Object])
    ], ActionStateStoreImpl);
    return ActionStateStoreImpl;
}());
exports.ActionStateStoreImpl = ActionStateStoreImpl;
