"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var r = require("resul-ts");
var inversify_1 = require("inversify");
var H = require("../di_interfaces");
var di_symbols_1 = require("../di_symbols");
var block_xml_1 = require("./block_xml");
var block_config_1 = require("./block_config");
var domain_functions_1 = require("./domain_functions");
var event_1 = require("./event");
var BasicBlockProviderFactoryImpl = (function () {
    function BasicBlockProviderFactoryImpl(config, day_names, event_bus, ohno, runtime_data, runtime_manager, bxbf) {
        this.config = config;
        this.day_names = day_names;
        this.event_bus = event_bus;
        this.ohno = ohno;
        this.runtime_data = runtime_data;
        this.runtime_manager = runtime_manager;
        this.bxbf = bxbf;
    }
    BasicBlockProviderFactoryImpl.prototype.runtime_provider = function () {
        return {
            namespace: function () { return ''; },
            domain_functions: domain_functions_1.get_domain_functions(this.runtime_manager, this.runtime_data, this.event_bus, this.ohno, this.day_names, this.config),
            action_types: event_1.get_action_specs,
            block_metadata: {
                restart_when_finished: ['self_listen', 'when'],
                finish_out_of_run_group: ['on_running_group_activated'],
            },
        };
    };
    BasicBlockProviderFactoryImpl.prototype.block_provider_and_xml = function (block_config_deps) {
        var _this = this;
        var runtime_provider = this.runtime_provider();
        var block_provider = tslib_1.__assign({}, runtime_provider, { config: block_config_1.get_block_config(block_config_deps) });
        return {
            get_block_provider: function () { return block_provider; },
            get_default_block_xml: function () {
                if (_this.bxbf == undefined) {
                    return r.fail('Heart workspace requirements unfulfilled, cannot get BlockXML for basic blocks.');
                }
                var bxml_builder = _this.bxbf.create(block_provider);
                block_xml_1.apply_basic_block_defaults(bxml_builder);
                return r.success(bxml_builder.get_block_xml());
            },
        };
    };
    BasicBlockProviderFactoryImpl = tslib_1.__decorate([
        inversify_1.injectable(),
        tslib_1.__param(0, inversify_1.inject(di_symbols_1.BINDING.Config)),
        tslib_1.__param(1, inversify_1.inject(di_symbols_1.BINDING.DayNames)),
        tslib_1.__param(2, inversify_1.inject(di_symbols_1.BINDING.EventBus)),
        tslib_1.__param(3, inversify_1.inject(di_symbols_1.BINDING.Ohno)),
        tslib_1.__param(4, inversify_1.inject(di_symbols_1.BINDING.RuntimeData)),
        tslib_1.__param(5, inversify_1.inject(di_symbols_1.BINDING.RuntimeManager)),
        tslib_1.__param(6, inversify_1.inject(di_symbols_1.BINDING.BlockXmlBuilderFactory)), tslib_1.__param(6, inversify_1.optional()),
        tslib_1.__metadata("design:paramtypes", [Object, Object, Object, Object, Object, Object, Object])
    ], BasicBlockProviderFactoryImpl);
    return BasicBlockProviderFactoryImpl;
}());
exports.BasicBlockProviderFactoryImpl = BasicBlockProviderFactoryImpl;
