import * as r from 'resul-ts';
import { Catastrophe } from 'catastrophic';
import { ProcedureDefinitionBlock, CompiledBlock } from './block_types';
export { CompiledBlock };
export declare type clog = typeof console.log;
export interface Logger {
    fatal: clog;
    error: clog;
    warn: clog;
    info: clog;
    debug: clog;
    trace: clog;
}
export interface DayNames {
    0: string;
    1: string;
    2: string;
    3: string;
    4: string;
    5: string;
    6: string;
}
export declare type Dict<T> = {
    [key: string]: T;
};
export declare type ID = string;
export declare type URL = string;
export declare type MilliSeconds = number;
export interface Identities {
    typeclass_id: ID;
    source_map_entity: ID;
    source_map_rbid: ID;
    target_entity: ID;
    interpreter_id: ID;
}
export interface InterpreterPriorities {
    readonly creation_counter: number;
    readonly frame_created: number;
    readonly responder_priority: number;
}
export interface InterpreterMetadata {
    readonly original_identities: Readonly<Identities>;
    readonly priorities: InterpreterPriorities;
    readonly typeclass_id: string;
    readonly interpreter_id: ID;
    readonly type: string;
    readonly group_id: string | undefined;
}
export interface Entity {
    id: ID;
    blocksXML?: string;
}
export declare enum EntityState {
    Unknown = 0,
    Known = 1,
    Destructing = 2,
    Disposed = 3,
}
export interface EntityCompileResult {
    compiled_block_map: CompiledBlockMap;
    procedures: {
        [function_name: string]: ProcedureDefinitionBlock;
    };
}
export interface ProcedureContainer {
    script: ProcedureDefinitionBlock;
    source_entity_id: ID;
    name: string;
}
export interface UncompiledEntity extends Entity {
    blocksXML: string;
}
export interface CompiledEntity extends EntityCompileResult, UncompiledEntity {
    running_group_id?: {
        [compiled_block_map_rbid: string]: ID | undefined;
    };
}
export declare type MaybeCompiledEntities = r.Result<CompiledEntity[], Catastrophe>;
export interface CompiledBlockMap {
    [id: string]: CompiledBlock;
}
export declare type Storable = string | number | boolean;
export interface BaseVariableSpec {
    type: string;
    id: ID;
    value: Storable | Storable[];
}
export declare enum VariableScope {
    global = 0,
    script = 1,
    entity = 2,
}
export interface GlobalVariableSpec extends BaseVariableSpec {
    scope: VariableScope.global;
}
export interface ScriptVariableSpec extends BaseVariableSpec {
    scope: VariableScope.script;
}
export interface EntityVariableSpec extends BaseVariableSpec {
    scope: VariableScope.entity;
    entity_id: ID;
}
export declare type VariableSpec = ScriptVariableSpec | EntityVariableSpec | GlobalVariableSpec;
export declare type VariableSpecDict = {
    [var_id: string]: VariableSpec;
};
export declare type ScriptVariableSpecDict = {
    [var_id: string]: ScriptVariableSpec;
};
export declare type VariableStore = {
    [var_id: string]: any;
};
export declare type EntityVariableStore = {
    [entity_id: string]: VariableStore;
};
export declare enum StepResult {
    yielding = 0,
    finished = 1,
}
export declare type ForcedBlockIds = {
    [block_id: string]: boolean;
};
export interface ClientErrorProperties {
    namespace: string;
    id: string;
    native_error?: Error;
    [k: string]: any;
}
export interface Task {
    entity_id: ID;
    interpreter_id: ID;
    lifetime?: MilliSeconds;
    blocking: boolean;
    on_start?: () => void;
    on_tick?: (delta_time?: MilliSeconds, amount_done?: number) => void;
    on_end?: (delta_time?: MilliSeconds) => void;
}
export interface TaskHandle {
    stop: () => void;
}
export interface RuntimeStackMetadata {
    interpreter_id: ID;
    source_map_rbid: ID;
    source_entity_id: ID;
    block_id?: ID;
    proc_id?: ID;
    proc_parameters?: {
        [id: string]: any;
    };
}
