import * as BLK from './block_types';
import { Dict, ID, Identities, InterpreterPriorities, ScriptVariableSpecDict } from './basic_types';
import { BlockInterpreter, BlockInterpreterFactory, BlockPool, BlockRegistry, EventBusPrivate, Ohno, OptiCompiler, RuntimeManager, Util, OnInterpreterFinished } from './di_interfaces';
export interface ResultProcedureCall {
    kind: 'procedure_call';
    call_arguments: any;
    yield_to_block: BLK.CompiledBlock;
    procedure_block: BLK.ProcedureDefinitionBlock;
    call_block?: BLK.ProcedureCallReturnBlock;
}
export interface ResultProcedureReturn {
    kind: 'procedure_return';
    return_value?: any;
}
export interface ResultRunChild {
    kind: 'run_child';
    child_block: BLK.CompiledBlock | undefined;
}
export interface EmptyResult {
    kind: 'empty';
}
export interface NullOrUndefinedErrorResult {
    kind: 'null_or_undefined_error';
}
export interface EnterWarp {
    kind: 'enter_warp';
    child_block: BLK.CompiledBlock | undefined;
}
export interface ResultValue {
    kind: 'value';
    value: any;
}
export declare type BlockResult = EmptyResult | ResultValue | ResultRunChild | ResultProcedureCall | ResultProcedureReturn | NullOrUndefinedErrorResult | EnterWarp;
export declare class BlockInterpreterFactoryImpl implements BlockInterpreterFactory {
    private u;
    private ohno;
    private event_bus;
    private block_registry;
    private opti_compiler;
    private block_pool;
    private block_runner?;
    private should_report_current_running_block;
    private max_procedure_calls_per_interpreter_step;
    private max_warp_iterations_per_interpreter_step;
    private max_call_stack_size;
    constructor(u: Util, ohno: Ohno, event_bus: EventBusPrivate, block_registry: BlockRegistry, opti_compiler: OptiCompiler, block_pool: BlockPool);
    create(run_mgr: RuntimeManager, identities: Identities, priorities: InterpreterPriorities, compile_cache_id: ID, compiled_block: BLK.CompiledBlock, script_variable_specs: ScriptVariableSpecDict, group_id: ID | undefined, is_warped: boolean, action_parameters?: Dict<any>, on_finished?: OnInterpreterFinished): BlockInterpreter;
    clear(): void;
}
