"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var _clone = require("lodash/clone");
var inversify_1 = require("inversify");
var di_symbols_1 = require("./di_symbols");
var H = require("./di_interfaces");
var BlockPoolImpl = (function () {
    function BlockPoolImpl(event_bus, log, ohno, u, config) {
        this.event_bus = event_bus;
        this.log = log;
        this.ohno = ohno;
        this.u = u;
        this.config = config;
        this.pool = [];
        var size = config.get().block_pool_preallocation_size;
        var a = [];
        for (var i = 0; i < size; i++) {
            a.push(this.create());
        }
        for (var i = 0; i < size; i++) {
            this.release(a.pop());
        }
    }
    BlockPoolImpl.prototype.create = function () {
        return {
            params: {},
            kind: '',
            type: '',
            id: '',
            parent_block: undefined,
            next_block: undefined,
            child_block: [],
            first_evaluation: true,
            done_evaluating: false,
            output_type: '',
            last_call: undefined,
            waiting_for_procedure: undefined,
            disabled: false,
            conditions: [],
            procedure_name: '',
            procedure_return_value: undefined,
            times_left: 0,
        };
    };
    BlockPoolImpl.prototype.release_block_param = function (b) {
        if (b == undefined) {
            return;
        }
        if (this.u.block.is.compiled_block(b)) {
            this.release(b);
        }
    };
    BlockPoolImpl.prototype.reset = function (block) {
        this.release_block_param(block.next_block);
        for (var i = 0; i < block.child_block.length; i++) {
            this.release_block_param(block.child_block[i]);
        }
        for (var i = 0; i < block.conditions.length; i++) {
            this.release_block_param(block.conditions[i]);
        }
        for (var param in block.params) {
            this.release_block_param(block.params[param]);
        }
        block.params = {};
        block.kind = '';
        block.type = '';
        block.id = '';
        block.parent_block = undefined;
        block.next_block = undefined;
        block.child_block = [];
        block.first_evaluation = true;
        block.done_evaluating = false;
        block.output_type = '';
        block.last_call = undefined;
        block.waiting_for_procedure = undefined;
        block.disabled = false;
        block.conditions = [];
        block.procedure_name = '';
        block.procedure_return_value = undefined;
        block.times_left = 0;
    };
    BlockPoolImpl.prototype.clone = function (block, par) {
        var c = this.get();
        var orig = block;
        for (var param_name in orig.params) {
            var param = orig.params[param_name];
            if (param === undefined) {
                c.params[param_name] = undefined;
            }
            else {
                if (this.u.block.is.compiled_block(param)) {
                    c.params[param_name] = this.clone(param, c);
                }
                else {
                    c.params[param_name] = _clone(param);
                }
            }
        }
        c.kind = orig.kind;
        c.type = orig.type;
        c.id = orig.id;
        c.parent_block = par;
        if (orig.next_block != undefined) {
            c.next_block = this.clone(orig.next_block, par);
        }
        c.child_block = [];
        for (var i = 0; i < orig.child_block.length; i++) {
            var maybe_child = orig.child_block[i];
            if (maybe_child == undefined) {
                c.child_block.push(undefined);
            }
            else {
                c.child_block.push(this.clone(maybe_child, c));
            }
        }
        c.first_evaluation = orig.first_evaluation;
        c.done_evaluating = orig.done_evaluating;
        c.output_type = orig.output_type;
        c.last_call = orig.last_call;
        c.waiting_for_procedure = orig.waiting_for_procedure;
        c.disabled = orig.disabled;
        for (var i = 0; i < orig.conditions.length; i++) {
            var cond = orig.conditions[i];
            if (cond == undefined) {
                c.conditions.push(undefined);
            }
            else {
                c.conditions.push(this.clone(cond, c));
            }
        }
        c.procedure_name = orig.procedure_name;
        c.procedure_return_value = orig.procedure_return_value;
        c.times_left = orig.times_left;
        return c;
    };
    BlockPoolImpl.prototype.get = function () {
        var block = this.pool.pop();
        if (block == undefined) {
            return this.create();
        }
        return block;
    };
    BlockPoolImpl.prototype.release = function (block) {
        this.reset(block);
        this.pool.push(block);
    };
    BlockPoolImpl = tslib_1.__decorate([
        inversify_1.injectable(),
        tslib_1.__param(0, inversify_1.inject(di_symbols_1.BINDING.EventBus)),
        tslib_1.__param(1, inversify_1.inject(di_symbols_1.BINDING.Log)),
        tslib_1.__param(2, inversify_1.inject(di_symbols_1.BINDING.Ohno)),
        tslib_1.__param(3, inversify_1.inject(di_symbols_1.BINDING.Util)),
        tslib_1.__param(4, inversify_1.inject(di_symbols_1.BINDING.Config)),
        tslib_1.__metadata("design:paramtypes", [Object, Object, Object, Object, Object])
    ], BlockPoolImpl);
    return BlockPoolImpl;
}());
exports.BlockPoolImpl = BlockPoolImpl;
