"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var inversify_1 = require("inversify");
var di_symbols_1 = require("./di_symbols");
var B = require("./block_types");
var check_domain_block_optimization = { domain_block: true };
var BlockPredicatesImpl = (function () {
    function BlockPredicatesImpl(block_registry, ohno, log) {
        this.block_registry = block_registry;
        this.ohno = ohno;
        this.log = log;
    }
    BlockPredicatesImpl.prototype.responder_block = function (b) {
        return this.block_registry.has_responder_type(b.type);
    };
    BlockPredicatesImpl.prototype.responder_type = function (t) {
        return this.block_registry.has_responder_type(t);
    };
    BlockPredicatesImpl.prototype.atomic_type = function (t) {
        return B.ATOMIC_BLOCKS[t] !== undefined;
    };
    BlockPredicatesImpl.prototype.async_tell = function (b) {
        return B.SPECIAL_BLOCKS.tell == B.SPECIAL_BLOCKS[b.type];
    };
    BlockPredicatesImpl.prototype.sync_tell = function (b) {
        return B.SPECIAL_BLOCKS.sync_tell == B.SPECIAL_BLOCKS[b.type];
    };
    BlockPredicatesImpl.prototype.warp = function (b) {
        return B.SPECIAL_BLOCKS.warp == B.SPECIAL_BLOCKS[b.type];
    };
    BlockPredicatesImpl.prototype.logic_empty = function (b) {
        return B.SPECIAL_BLOCKS.logic_empty == B.SPECIAL_BLOCKS[b.type];
    };
    BlockPredicatesImpl.prototype.special_block = function (b) {
        return B.SPECIAL_BLOCKS[b.kind];
    };
    BlockPredicatesImpl.prototype.domain_block = function (b) {
        return check_domain_block_optimization[b.kind];
    };
    BlockPredicatesImpl.prototype.loop_block = function (b) {
        return B.LOOP_BLOCKS[b.kind];
    };
    BlockPredicatesImpl.prototype.cond_block = function (b) {
        return B.IF_BLOCKS[b.kind];
    };
    BlockPredicatesImpl.prototype.proc_block = function (b) {
        return B.PROCEDURE_BLOCKS[b.kind];
    };
    BlockPredicatesImpl.prototype.controls_if = function (b) {
        return B.IF_BLOCKS['controls_if'] == B.IF_BLOCKS[b.type];
    };
    BlockPredicatesImpl.prototype.controls_if_no_else = function (b) {
        return B.IF_BLOCKS.controls_if_no_else == B.IF_BLOCKS[b.type];
    };
    BlockPredicatesImpl.prototype.event_block = function (b) {
        return B.EVENT_BLOCKS[b.type] || this.block_registry.block_restart_when_finished(b.type);
    };
    BlockPredicatesImpl.prototype.repeat_n_times = function (b) {
        return B.LOOP_BLOCKS.repeat_n_times == B.LOOP_BLOCKS[b.type];
    };
    BlockPredicatesImpl.prototype.repeat_forever = function (b) {
        return B.LOOP_BLOCKS.repeat_forever == B.LOOP_BLOCKS[b.type];
    };
    BlockPredicatesImpl.prototype.repeat_forever_until = function (b) {
        return B.LOOP_BLOCKS.repeat_forever_until == B.LOOP_BLOCKS[b.type];
    };
    BlockPredicatesImpl.prototype.wait_until = function (b) {
        return B.LOOP_BLOCKS.wait_until == B.LOOP_BLOCKS[b.type];
    };
    BlockPredicatesImpl.prototype.break = function (b) {
        return B.LOOP_BLOCKS.break == B.LOOP_BLOCKS[b.type];
    };
    BlockPredicatesImpl.prototype.procedures_defnoreturn = function (b) {
        return B.PROCEDURE_BLOCKS.procedures_defnoreturn == B.PROCEDURE_BLOCKS[b.type];
    };
    BlockPredicatesImpl.prototype.procedures_callreturn = function (b) {
        return B.PROCEDURE_BLOCKS.procedures_callreturn == B.PROCEDURE_BLOCKS[b.type];
    };
    BlockPredicatesImpl.prototype.procedures_callnoreturn = function (b) {
        return B.PROCEDURE_BLOCKS.procedures_callnoreturn == B.PROCEDURE_BLOCKS[b.type];
    };
    BlockPredicatesImpl.prototype.procedures_return_value = function (b) {
        return B.PROCEDURE_BLOCKS.procedures_return_value == B.PROCEDURE_BLOCKS[b.type];
    };
    BlockPredicatesImpl.prototype.procedures_parameter = function (b) {
        return B.PROCEDURE_BLOCKS.procedures_parameter == B.PROCEDURE_BLOCKS[b.type];
    };
    BlockPredicatesImpl.prototype.compiled_block = function (b) {
        return b !== undefined && b.id != undefined;
    };
    BlockPredicatesImpl.prototype.atomic = function (b) {
        return b !== undefined && !this.compiled_block(b);
    };
    BlockPredicatesImpl = tslib_1.__decorate([
        inversify_1.injectable(),
        tslib_1.__param(0, inversify_1.inject(di_symbols_1.BINDING.BlockRegistry)),
        tslib_1.__param(1, inversify_1.inject(di_symbols_1.BINDING.Ohno)),
        tslib_1.__param(2, inversify_1.inject(di_symbols_1.BINDING.Log)),
        tslib_1.__metadata("design:paramtypes", [Object, Object, Object])
    ], BlockPredicatesImpl);
    return BlockPredicatesImpl;
}());
exports.BlockPredicatesImpl = BlockPredicatesImpl;
