import { Blockly } from './blockly_interface';
import { ID, Dict } from './basic_types';
import * as P from './public_interfaces';
export interface DomainFunctionArgs {
    [k: string]: any;
}
export interface DomainFunctionInternals {
    runtime_manager: P.RuntimeManager;
    add_user_procedure_call_to_stack(function_id: ID, target_entity_id: string, paramaters: {
        [param_id: string]: any;
    }): void;
    get_action_parameter(parameter_id: string): undefined | any;
}
export declare type UserProcedureCall = DomainFunctionInternals['add_user_procedure_call_to_stack'];
export declare type DomainFunction = (args: DomainFunctionArgs, interpreter_id: ID, target_entity: ID, internals: DomainFunctionInternals) => any;
export interface FunctionDict {
    [function_name: string]: DomainFunction;
}
export interface BlockConfig {
    message0: string;
    args0: BlockConfigArg[];
    lastDummyAlign0?: BCAAlignment;
    message1?: string;
    args1?: BlockConfigArg[];
    lastDummyAlign1?: BCAAlignment;
    message2?: string;
    args2?: BlockConfigArg[];
    lastDummyAlign2?: BCAAlignment;
    message3?: string;
    args3?: BlockConfigArg[];
    lastDummyAlign3?: BCAAlignment;
    message4?: string;
    args4?: BlockConfigArg[];
    lastDummyAlign4?: BCAAlignment;
    message5?: string;
    args5?: BlockConfigArg[];
    lastDummyAlign5?: BCAAlignment;
    message6?: string;
    args6?: BlockConfigArg[];
    lastDummyAlign6?: BCAAlignment;
    colour: string | number;
    helpUrl?: string;
    init?: Function;
    inputsInline?: boolean;
    mutator?: string;
    nextStatement?: boolean | null;
    output?: string | string[] | null;
    previousStatement?: boolean | null;
    tooltip?: string;
    type?: string;
    extensions?: string[];
}
export interface BlockConfigDict {
    [block_name: string]: BlockConfig;
}
export interface BlockConfigArgBase {
    align?: BCAAlignment;
    check?: string | string[];
    variableTypes?: string[];
}
export declare type BCAOptionTuple = [string, string];
export declare type BCAOptionList = BCAOptionTuple[];
export declare type BCAOptionFunction = () => BCAOptionList;
export declare type BCAAlignment = 'LEFT' | 'CENTRE' | 'RIGHT';
export declare type BlockConfigArg = InputValue | InputDummy | InputStatement | FieldInput | FieldDropdown | FieldCheckbox | FieldColour | FieldNumber | FieldAngle | FieldVariable | FieldDate | FieldLabel | FieldImage | FieldIcon;
export interface InputValue extends BlockConfigArgBase {
    name: string;
    type: 'input_value';
    alt?: BlockConfigArg;
}
export interface InputDummy extends BlockConfigArgBase {
    type: 'input_dummy';
    alt?: BlockConfigArg;
}
export interface InputStatement extends BlockConfigArgBase {
    name: string;
    type: 'input_statement';
    alt?: BlockConfigArg;
}
export interface FieldInput extends BlockConfigArgBase {
    name: string;
    type: 'field_input';
    spellcheck?: boolean;
    text: string;
    alt?: BlockConfigArg;
}
export interface FieldDropdown extends BlockConfigArgBase {
    name: string;
    type: 'field_dropdown';
    options: BCAOptionList | BCAOptionFunction;
    alt?: BlockConfigArg;
}
export interface FieldCheckbox extends BlockConfigArgBase {
    name: string;
    type: 'field_checkbox';
    checked: boolean;
    alt?: BlockConfigArg;
}
export interface FieldColour extends BlockConfigArgBase {
    name: string;
    type: 'field_colour';
    colour: string;
    alt?: BlockConfigArg;
}
export interface FieldNumber extends BlockConfigArgBase {
    name: string;
    type: 'field_number';
    value: number;
    min?: number;
    max?: number;
    precision?: number;
    alt?: BlockConfigArg;
}
export interface FieldAngle extends BlockConfigArgBase {
    name: string;
    type: 'field_angle';
    angle: string;
    alt?: BlockConfigArg;
}
export interface FieldVariable extends BlockConfigArgBase {
    name: string;
    type: 'field_variable';
    variable: string;
    alt?: BlockConfigArg;
}
export interface FieldDate extends BlockConfigArgBase {
    name: string;
    type: 'field_date';
    date: string;
    alt?: BlockConfigArg;
}
export interface FieldLabel extends BlockConfigArgBase {
    type: 'field_label';
    text: string;
    class: string;
    alt?: BlockConfigArg;
}
export interface FieldImage extends BlockConfigArgBase {
    type: 'field_image';
    src: string;
    width: number;
    height: number;
    alt?: BlockConfigArg | string;
}
export interface FieldIcon extends BlockConfigArgBase {
    type: 'field_icon';
    src: string;
    width: number;
    height: number;
    is_head: boolean;
    alt?: BlockConfigArg | string;
}
export interface ActionSpec {
    id: string;
    entity_specific: boolean;
    responder_blocks: ResponderSpec[];
    statefulness?: ActionStateSpec;
}
export declare enum ResponderType {
    Action = "action",
    State = "state",
    Dynamic = "dynamic",
    Constructor = "constructor",
    Destructor = "destructor",
}
export declare type TriggerFunc = (actions: Action[], value?: string, sub_type?: string, entity_id?: ID) => boolean;
export interface ResponderSpec {
    id: string;
    type: ResponderType;
    async: boolean;
    priority?: number;
    filter_arg_names?: {
        value: string;
        sub_type: string;
    };
    trigger_function?: TriggerFunc;
}
export interface ResponderInfo {
    namespace: string;
    action_spec: ActionSpec;
    responder_spec: ResponderSpec;
}
export interface ActionStateSpec {
    default_value: string;
    automatic_transitions: StateTransitionTable | 'one_frame';
    use_sub_type: boolean;
}
export interface StateTransitionTable {
    [current_state: string]: string;
}
export interface Action {
    entity_id?: ID;
    id: string;
    namespace: string;
    parameters?: Dict<any>;
    value?: string;
    sub_type?: string;
}
export declare type BlockConfigFactory = (blockly: Blockly) => BlockConfigDict;
export declare type FunctionDictFactory = () => FunctionDict;
export interface RuntimeProvider {
    namespace(): string;
    action_types(): ActionSpec[];
    domain_functions: FunctionDictFactory;
    block_metadata?: {
        restart_when_finished?: string[];
        finish_out_of_run_group?: string[];
    };
}
export interface BlockProvider extends RuntimeProvider {
    config: BlockConfigFactory;
}
export interface ParsedFullName {
    namespace: string;
    function_id: string;
}
export declare function parse_namespaced_id(ns_id: string): ParsedFullName;
export declare function namespaced_id(namespace: string, function_name: string): string;
