export declare enum LOOP_BLOCKS {
    repeat_forever = 1,
    repeat_n_times = 2,
    repeat_forever_until = 3,
    wait_until = 4,
    break = 5,
}
export declare enum IF_BLOCKS {
    controls_if = 1,
    controls_if_no_else = 2,
}
export declare enum EVENT_BLOCKS {
    self_listen = 1,
    start_as_a_mirror = 2,
    player_position_on_change = 3,
    mouse_on_emit = 4,
    block_on_break = 5,
    when = 6,
}
export declare enum PROCEDURE_BLOCKS {
    procedures_defnoreturn = 1,
    procedures_callnoreturn = 2,
    procedures_callreturn = 3,
    procedures_return_value = 4,
    procedures_parameter = 5,
}
export declare enum MIDI_BLOCKS {
    midi_play_note = 1,
    midi_wait = 2,
}
export declare enum HAT_BLOCKS {
    start_on_click = 1,
    self_on_tap = 2,
    backdrop_on_change = 3,
    self_listen = 4,
    start_as_a_mirror = 5,
    procedures_defnoreturn = 6,
    on_swipe = 7,
    player_position_on_change = 8,
    mouse_on_emit = 9,
    block_on_break = 10,
    when = 11,
    on_running_group_activated = 12,
}
export declare enum ATOMIC_BLOCKS {
    math_number = 1,
    text = 2,
}
export declare enum SPECIAL_BLOCKS {
    tell = 1,
    logic_empty = 2,
    sync_tell = 3,
    warp = 4,
}
export declare enum BlockOutputType {
    none = 0,
    any = 1,
    number = 2,
    list = 3,
    string = 4,
}
export declare type BlockParam = CompiledBlock | string | number | boolean | undefined;
export declare type MaybeBlock = CompiledBlock | undefined;
export interface PreBlock {
    type: string;
    kind: string;
    id: string;
    params: {
        [p_id: string]: BlockParam;
    };
    parent_block?: CompiledBlock;
    next_block?: CompiledBlock;
    child_block: MaybeBlock[];
    first_evaluation: boolean;
    done_evaluating: boolean;
    disabled: boolean;
    output_type?: BlockOutputType;
}
export interface Block extends PreBlock {
    waiting_for_procedure?: boolean;
    last_call?: number;
}
export interface ConditionalBlock extends Block {
    conditions: MaybeBlock[];
}
export interface IfBlock extends ConditionalBlock {
    kind: 'controls_if';
}
export interface IfNoElseBlock extends ConditionalBlock {
    kind: 'controls_if_no_else';
}
export interface EventBlock extends Block {
    kind: 'event_block';
}
export interface WhenBlock extends Block {
    kind: 'when';
}
export interface RepeatNTimesBlock extends Block {
    kind: 'repeat_n_times';
    times_left: number;
}
export interface RepeatForeverBlock extends Block {
    kind: 'repeat_forever';
}
export interface RepeatForeverUntilBlock extends Block {
    kind: 'repeat_forever_until';
}
export interface WaitUntilBlock extends Block {
    kind: 'wait_until';
}
export interface BreakBlock extends Block {
    kind: 'break';
}
export interface ProcedureDefinitionBlock extends Block {
    kind: 'procedures_defnoreturn';
    procedure_name: string;
}
export interface ProcedureCallReturnBlock extends Block {
    kind: 'procedures_callreturn';
    procedure_name: string;
    procedure_return_value?: any;
}
export interface ProcedureCallNoReturnBlock extends Block {
    kind: 'procedures_callnoreturn';
    procedure_name: string;
}
export interface ProcedureReturnValueBlock extends Block {
    kind: 'procedures_return_value';
}
export interface ProcedureParameterBlock extends Block {
    kind: 'procedures_parameter';
}
export interface TellBlock extends Block {
    kind: 'tell';
}
export interface SyncTellBlock extends Block {
    kind: 'sync_tell';
}
export interface WarpBlock extends Block {
    kind: 'warp';
}
export interface LogicEmptyBlock extends Block {
    kind: 'logic_empty';
}
export interface DomainBlock extends Block {
    kind: 'domain_block';
}
export interface ResponderBlock extends Block {
    kind: 'responder_block';
}
export declare type LoopBlock = RepeatNTimesBlock | RepeatForeverBlock | RepeatForeverUntilBlock | WaitUntilBlock | BreakBlock;
export declare type CondBlock = IfBlock | IfNoElseBlock;
export declare type SpecialBlock = TellBlock | SyncTellBlock | LogicEmptyBlock | WarpBlock;
export declare type ProcBlock = ProcedureDefinitionBlock | ProcedureCallReturnBlock | ProcedureCallNoReturnBlock | ProcedureReturnValueBlock | ProcedureParameterBlock;
export declare type CompiledBlock = EventBlock | ResponderBlock | ProcBlock | CondBlock | LoopBlock | DomainBlock | SpecialBlock;
export declare type Expression = LogicEmptyBlock | ProcedureParameterBlock | ProcedureCallReturnBlock | DomainBlock;
export declare type Statement = EventBlock | ResponderBlock | ProcedureDefinitionBlock | ProcedureCallNoReturnBlock | ProcedureReturnValueBlock | CondBlock | LoopBlock | TellBlock | SyncTellBlock | WarpBlock | Expression | WhenBlock;
export declare type PoolBlock = EventBlock & ResponderBlock & DomainBlock & IfBlock & IfNoElseBlock & RepeatNTimesBlock & RepeatForeverBlock & RepeatForeverUntilBlock & WaitUntilBlock & BreakBlock & ProcedureDefinitionBlock & ProcedureCallReturnBlock & ProcedureCallNoReturnBlock & ProcedureReturnValueBlock & ProcedureParameterBlock & WhenBlock & TellBlock & SyncTellBlock & WarpBlock & LogicEmptyBlock;
