"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _defaultsDeep = require("lodash/defaultsDeep");
var inversify_1 = require("inversify");
var di_symbols_1 = require("./di_symbols");
var action_state_store_1 = require("./action_state_store");
var basic_blocks_1 = require("./basic_blocks");
var block_interpreter_1 = require("./block_interpreter");
var block_pool_1 = require("./block_pool");
var block_predicates_1 = require("./block_predicates");
var block_registry_1 = require("./block_registry");
var block_util_1 = require("./block_util");
var block_xml_1 = require("./block_xml");
var config_1 = require("./config");
var day_names_1 = require("./basic_blocks/day_names");
var dom_compiler_1 = require("./dom_compiler");
var html_compiler_1 = require("./html_compiler");
var compiler_1 = require("./opti/compiler");
var program_cache_1 = require("./opti/program_cache");
var factory_1 = require("./opti/factory");
var prng_factory_1 = require("./prng_factory");
var runtime_data_1 = require("./runtime_data");
var runtime_manager_1 = require("./runtime_manager");
var task_manager_1 = require("./task_manager");
var index_1 = require("./test_blocks/index");
var index_2 = require("./benchmark_blocks/index");
var assert_1 = require("./test_blocks/assert");
var toolbox_1 = require("./toolbox");
var util_1 = require("./util");
var error_types_1 = require("./error_types");
var event_bus_1 = require("./event/event_bus");
var DEFAULT_BASIC_BLOCKS_REQUIREMENTS = {
    day_names: day_names_1.DEFAULT_DAY_NAMES,
};
function get_instance(spec) {
    spec.basic_blocks_requirements = _defaultsDeep(spec.basic_blocks_requirements, DEFAULT_BASIC_BLOCKS_REQUIREMENTS);
    var cont = new inversify_1.Container();
    cont.bind(di_symbols_1.BINDING.Log)
        .toConstantValue(spec.logger);
    var event_bus = event_bus_1.create_event_bus();
    cont.bind(di_symbols_1.BINDING.EventBus)
        .toConstantValue(event_bus);
    cont.bind(di_symbols_1.BINDING.Config)
        .to(config_1.ConfigImpl)
        .inSingletonScope();
    var config = cont.get(di_symbols_1.BINDING.Config);
    config.set(spec.configuration);
    if (spec.workspace_requirements != undefined) {
        var blockly = spec.workspace_requirements.blockly;
        cont.bind(di_symbols_1.BINDING.Blockly)
            .toConstantValue(blockly);
        cont.bind(di_symbols_1.BINDING.BlockXmlBuilderFactory)
            .to(block_xml_1.BlockXMLBuilderFactoryImpl)
            .inSingletonScope();
    }
    var has_dom_parser = spec.compiler_requirements && spec.compiler_requirements.dom_parser;
    var has_html_parser = spec.compiler_requirements && spec.compiler_requirements.html_parser;
    if (has_dom_parser) {
        cont.bind(di_symbols_1.BINDING.DOMParser)
            .toConstantValue(spec.compiler_requirements.dom_parser);
        cont.bind(di_symbols_1.BINDING.Compiler)
            .to(dom_compiler_1.DOMParserCompilerImpl)
            .inSingletonScope();
    }
    else if (has_html_parser) {
        cont.bind(di_symbols_1.BINDING.HtmlParser)
            .toConstantValue(spec.compiler_requirements.html_parser);
        cont.bind(di_symbols_1.BINDING.Compiler)
            .to(html_compiler_1.HtmlParserCompilerImpl)
            .inSingletonScope();
    }
    cont.bind(di_symbols_1.BINDING.DayNames)
        .toConstantValue(spec.basic_blocks_requirements.day_names);
    cont.bind(di_symbols_1.BINDING.Ohno)
        .toConstantValue(error_types_1.ohno);
    cont.bind(di_symbols_1.BINDING.ActionStateStore)
        .to(action_state_store_1.ActionStateStoreImpl)
        .inSingletonScope();
    cont.bind(di_symbols_1.BINDING.RuntimeData)
        .to(runtime_data_1.RuntimeDataImpl)
        .inSingletonScope();
    cont.bind(di_symbols_1.BINDING.BlockInterpreterFactory)
        .to(block_interpreter_1.BlockInterpreterFactoryImpl)
        .inSingletonScope();
    cont.bind(di_symbols_1.BINDING.OptiProgramCache)
        .to(program_cache_1.OptiProgramCacheImpl)
        .inSingletonScope();
    cont.bind(di_symbols_1.BINDING.OptiRunnerFactory)
        .to(factory_1.OptiRunnerFactory)
        .inSingletonScope();
    cont.bind(di_symbols_1.BINDING.BasicBlockProviderFactory)
        .to(basic_blocks_1.BasicBlockProviderFactoryImpl)
        .inSingletonScope();
    cont.bind(di_symbols_1.BINDING.PRNGFactory)
        .to(prng_factory_1.PRNGFactoryImpl)
        .inSingletonScope();
    cont.bind(di_symbols_1.BINDING.TestBlockProviderFactory)
        .to(index_1.TestBlockProviderFactoryImpl)
        .inSingletonScope();
    cont.bind(di_symbols_1.BINDING.BenchmarkBlockProviderFactory)
        .to(index_2.BenchmarkBlockProviderFactoryImpl)
        .inSingletonScope();
    cont.bind(di_symbols_1.BINDING.TestAssertionTool)
        .to(assert_1.TestAssertionToolImpl)
        .inSingletonScope();
    cont.bind(di_symbols_1.BINDING.ToolboxFactory)
        .to(toolbox_1.ToolboxFactoryImpl)
        .inSingletonScope();
    cont.bind(di_symbols_1.BINDING.BlockRegistry)
        .to(block_registry_1.BlockRegistryImpl)
        .inSingletonScope();
    cont.bind(di_symbols_1.BINDING.BlockPool)
        .to(block_pool_1.BlockPoolImpl)
        .inSingletonScope();
    cont.bind(di_symbols_1.BINDING.BlockPredicates)
        .to(block_predicates_1.BlockPredicatesImpl)
        .inSingletonScope();
    cont.bind(di_symbols_1.BINDING.OptiCompiler)
        .to(compiler_1.OptiCompilerImpl)
        .inSingletonScope();
    cont.bind(di_symbols_1.BINDING.RuntimeManager)
        .to(runtime_manager_1.RuntimeManagerImpl)
        .inSingletonScope();
    cont.bind(di_symbols_1.BINDING.TaskManager)
        .to(task_manager_1.TaskManagerImpl)
        .inSingletonScope();
    cont.bind(di_symbols_1.BINDING.BlockUtil)
        .to(block_util_1.BlockUtilImpl)
        .inSingletonScope();
    cont.bind(di_symbols_1.BINDING.Util)
        .to(util_1.UtilImpl)
        .inSingletonScope();
    return cont;
}
exports.get_instance = get_instance;
