"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _endsWith = require("lodash/endsWith");
var r = require("resul-ts");
var mutil = require("./migration_util");
var regex_extract_filename_ext = /([^/]*)\.([^/.]+)$/;
var Extract;
(function (Extract) {
    Extract[Extract["Name"] = 1] = "Name";
    Extract[Extract["Ext"] = 2] = "Ext";
})(Extract || (Extract = {}));
function xml_to_0(xml, filename) {
    var test_name_r = regex_extract_filename_ext.exec(filename);
    var test_name = filename;
    if (test_name_r != null && test_name_r.length >= 1) {
        test_name = test_name_r[Extract.Name];
    }
    return {
        version: 0,
        test_name: test_name,
        main_entity: {
            id: 'main',
            workspace_xml: xml,
            variables: [],
        },
        optional_entities: [],
        global_variables: mutil.get_variables_from_xml(xml),
    };
}
exports.xml_to_0 = xml_to_0;
function migrate_project(p) {
    return p;
}
exports.migrate_project = migrate_project;
function ht_to_heart_entity(e) {
    return {
        id: e.id,
        blocksXML: e.workspace_xml,
    };
}
function compile(project, heart) {
    var maybe_compiler = heart.get_compiler();
    if (r.is_error(maybe_compiler)) {
        return maybe_compiler;
    }
    var compiler = maybe_compiler.result;
    var entities = project.optional_entities.map(ht_to_heart_entity);
    entities.push(ht_to_heart_entity(project.main_entity));
    return compiler.compile(entities);
}
exports.compile = compile;
function get_projects(node_fs, dir) {
    var xml_filenames = node_fs.readdirSync(dir).filter(function (file) { return _endsWith(file, '.xml'); });
    var ht_filenames = node_fs.readdirSync(dir).filter(function (file) { return _endsWith(file, '.ht'); });
    var projects = [];
    for (var i = 0; i < xml_filenames.length; i++) {
        var filename = xml_filenames[i];
        var xml = node_fs.readFileSync(dir + filename, 'utf-8');
        projects.push(xml_to_0(xml, filename));
    }
    for (var i = 0; i < ht_filenames.length; i++) {
        var filename = ht_filenames[i];
        var json = node_fs.readFileSync(dir + filename, 'utf-8');
        projects.push(JSON.parse(json));
    }
    return projects.map(migrate_project);
}
exports.get_projects = get_projects;
