"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var r = require("resul-ts");
var _isString = require("lodash/isString");
var _unescape = require("lodash/unescape");
var catastrophic_1 = require("catastrophic");
var inversify_1 = require("inversify");
var B = require("./block_types");
var block_types_1 = require("./block_types");
var di_symbols_1 = require("./di_symbols");
var DOMParserCompilerImpl = (function () {
    function DOMParserCompilerImpl(block_pool, dom_parser, ohno, block, block_registry) {
        this.block_pool = block_pool;
        this.dom_parser = dom_parser;
        this.ohno = ohno;
        this.block = block;
        this.block_registry = block_registry;
    }
    DOMParserCompilerImpl.prototype.is_hat_block = function (block_type) {
        return block_types_1.HAT_BLOCKS[block_type]
            || this.block.is.responder_type(block_type)
            || this.block_registry.block_restart_when_finished(block_type);
    };
    DOMParserCompilerImpl.prototype.compile_entity = function (entity, force_compile_block_ids) {
        var xml = entity.blocksXML.trim();
        var w_json = this.compile_from_workspace_xml(xml, force_compile_block_ids);
        return tslib_1.__assign({}, entity, { procedures: w_json.procedures, compiled_block_map: w_json.compiled_block_map });
    };
    DOMParserCompilerImpl.prototype.compile = function (entities, force_compile_block_ids) {
        var _this = this;
        try {
            return r.success(entities.map(function (e) { return _this.compile_entity(e, force_compile_block_ids); }));
        }
        catch (e) {
            var err_msg = 'Heart could not compile one or more entities';
            if (e instanceof catastrophic_1.Catastrophe) {
                return r.error(err_msg, e);
            }
            else {
                return r.error(err_msg, this.ohno.compiler.system.unknown_compiler_error(e));
            }
        }
    };
    DOMParserCompilerImpl.prototype.compile_from_workspace_xml = function (xml_string, force_compile_block_ids) {
        var res = {
            procedures: {},
            compiled_block_map: {},
        };
        if (!xml_string) {
            return res;
        }
        var parsed_workspace = this.dom_parser.parseFromString("<kitten>" + xml_string + "</kitten>", 'text/xml');
        var dom = parsed_workspace.firstChild.firstChild;
        while (dom) {
            var j = this.xml_to_json(dom);
            dom = dom['nextElementSibling'];
            if (j == undefined) {
                continue;
            }
            if (force_compile_block_ids) {
                if (!force_compile_block_ids[j.id]) {
                    continue;
                }
            }
            else {
                if (!this.is_hat_block(j.type)) {
                    continue;
                }
            }
            if (this.block.is.procedures_defnoreturn(j)) {
                res.procedures[j.procedure_name] = j;
            }
            else {
                res.compiled_block_map[j.id] = j;
            }
        }
        return res;
    };
    DOMParserCompilerImpl.prototype.parse_xml_field = function (xml_dom) {
        if (xml_dom.getAttribute('name') == 'NUM') {
            return parseFloat(xml_dom.innerHTML);
        }
        return _unescape(xml_dom.innerHTML);
    };
    DOMParserCompilerImpl.prototype.xml_to_json = function (xml_dom, parent) {
        if (!xml_dom) {
            return undefined;
        }
        var child_nodes = {
            named: {},
            tagged: {},
        };
        for (var i = 0; i < xml_dom.children.length; i++) {
            var child = xml_dom.children[i];
            var tag_name = child.tagName;
            child_nodes.tagged[tag_name] = child;
            var name_1 = child.getAttribute('name');
            if (name_1) {
                child_nodes.named[name_1] = child;
            }
        }
        var next_block_dom = child_nodes.tagged['next'];
        var next_block_json = next_block_dom ? this.xml_to_json(next_block_dom.firstChild, parent) : undefined;
        var pre_block = this.block_pool.get();
        pre_block.kind = xml_dom.getAttribute('type');
        pre_block.type = xml_dom.getAttribute('type');
        pre_block.disabled = xml_dom.getAttribute('disabled');
        pre_block.id = xml_dom.getAttribute('id');
        pre_block.parent_block = parent;
        pre_block.next_block = next_block_json;
        pre_block.first_evaluation = true;
        pre_block.done_evaluating = false;
        pre_block.output_type = (xml_dom.parentNode && xml_dom.parentNode.tagName == 'value') ? B.BlockOutputType.number : B.BlockOutputType.none;
        for (var i = 0; i < xml_dom.children.length; i++) {
            var child = xml_dom.children[i];
            var child_name = child.getAttribute('name');
            if (child.tagName == 'value') {
                var param = this.xml_to_json(child.lastChild, pre_block);
                if (param != undefined) {
                    pre_block.params[child_name] = param;
                }
            }
            else if (child.tagName == 'field') {
                pre_block.params[child_name] = this.parse_xml_field(child);
            }
        }
        if (this.block.is.loop_block(pre_block)) {
            return this.loop_to_json(pre_block, child_nodes);
        }
        else if (this.block.is.cond_block(pre_block)) {
            return this.conditional_to_json(pre_block, child_nodes);
        }
        else if (this.block.is.event_block(pre_block)) {
            return this.event_to_json(pre_block, child_nodes);
        }
        else if (this.block.is.responder_block(pre_block)) {
            return this.responder_to_json(pre_block, child_nodes);
        }
        else if (this.block.is.proc_block(pre_block)) {
            return this.procedure_to_json(pre_block, child_nodes);
        }
        else if (this.block.is.async_tell(pre_block) || this.block.is.sync_tell(pre_block)) {
            return this.tell_to_json(pre_block, child_nodes);
        }
        else if (this.block.is.warp(pre_block)) {
            return this.warp_to_json(pre_block, child_nodes);
        }
        pre_block.kind = 'domain_block';
        return pre_block;
    };
    DOMParserCompilerImpl.prototype.responder_to_json = function (pre_block, child_nodes) {
        pre_block.kind = 'responder_block';
        var result = pre_block;
        if (child_nodes.tagged['statement']) {
            result.child_block.push(this.xml_to_json(child_nodes.tagged['statement'].lastChild, result));
        }
        return result;
    };
    DOMParserCompilerImpl.prototype.event_to_json = function (pre_block, child_nodes) {
        pre_block.kind = 'event_block';
        var result = pre_block;
        if (child_nodes.tagged['statement']) {
            result.child_block.push(this.xml_to_json(child_nodes.tagged['statement'].lastChild, result));
        }
        return result;
    };
    DOMParserCompilerImpl.prototype.loop_to_json = function (pre_block, child_nodes) {
        var named = child_nodes.named, tagged = child_nodes.tagged;
        var result = pre_block;
        if (this.block.is.repeat_n_times(result)
            || this.block.is.repeat_forever(result)
            || this.block.is.repeat_forever_until(result)) {
            result.child_block.push(named['DO'] ? this.xml_to_json(named['DO'].firstChild, result) : undefined);
        }
        return result;
    };
    DOMParserCompilerImpl.prototype.tell_to_json = function (pre_block, child_nodes) {
        var named = child_nodes.named, tagged = child_nodes.tagged;
        var result = pre_block;
        result.child_block.push(named['DO'] ? this.xml_to_json(named['DO'].firstChild, result) : undefined);
        return result;
    };
    DOMParserCompilerImpl.prototype.warp_to_json = function (pre_block, child_nodes) {
        var named = child_nodes.named, tagged = child_nodes.tagged;
        var result = pre_block;
        result.child_block.push(named['DO'] ? this.xml_to_json(named['DO'].firstChild, result) : undefined);
        return result;
    };
    DOMParserCompilerImpl.prototype.conditional_to_json = function (pre_block, child_nodes) {
        var named = child_nodes.named, tagged = child_nodes.tagged;
        var result = pre_block;
        result.params = {};
        result.conditions = [];
        function get_n_statements() {
            var keys = Object.keys(named);
            var max_if = 0;
            var max_do = 0;
            keys.forEach(function (key) {
                if (key.indexOf('IF') >= 0) {
                    max_if = Math.max(max_if, parseInt(key.split('IF')[1]));
                }
                else if (key.indexOf('DO') >= 0) {
                    max_do = Math.max(max_do, parseInt(key.split('DO')[1]));
                }
            });
            return Math.max(max_if, max_do) + 1;
        }
        var n_statements = get_n_statements();
        for (var i = 0; i < n_statements; ++i) {
            var condition = named['IF' + i];
            if (condition) {
                result.conditions.push(this.xml_to_json(condition.lastChild, pre_block));
            }
            else {
                result.conditions.push(undefined);
            }
            var statement = named['DO' + i];
            if (statement) {
                result.child_block.push(this.xml_to_json(statement.lastChild, pre_block));
            }
            else {
                result.child_block.push(undefined);
            }
        }
        var else_statement = named['ELSE'];
        if (else_statement) {
            result.child_block.push(this.xml_to_json(else_statement.lastChild, pre_block));
        }
        else {
            result.child_block.push(undefined);
        }
        return result;
    };
    DOMParserCompilerImpl.prototype.procedure_to_json = function (pre_block, child_nodes) {
        switch (B.PROCEDURE_BLOCKS[pre_block.type]) {
            case B.PROCEDURE_BLOCKS.procedures_defnoreturn: return this.procedure_definition_to_json(pre_block, child_nodes);
            case B.PROCEDURE_BLOCKS.procedures_callreturn: return this.procedure_call_return_to_json(pre_block, child_nodes);
            case B.PROCEDURE_BLOCKS.procedures_callnoreturn: return this.procedure_call_no_return_to_json(pre_block, child_nodes);
            case B.PROCEDURE_BLOCKS.procedures_return_value: return pre_block;
            case B.PROCEDURE_BLOCKS.procedures_parameter: return pre_block;
        }
        throw this.ohno.compiler.system.unknown_procedure_block_type({ pre_block: pre_block, child_nodes: child_nodes });
    };
    DOMParserCompilerImpl.prototype.procedure_definition_to_json = function (pre_block, child_nodes) {
        var named = child_nodes.named, tagged = child_nodes.tagged;
        var procedure_name = pre_block.params['NAME'];
        if (!_isString(procedure_name)) {
            throw this.ohno.compiler.system.procedure_name_not_string({ pre_block: pre_block });
        }
        var result = pre_block;
        result.procedure_name = procedure_name;
        result.params = {};
        var vars = tagged['mutation'];
        if (vars) {
            for (var i = 0; i < vars.children.length; i++) {
                var var_dom = vars.children[i];
                result.params[var_dom.getAttribute('name')] = true;
            }
        }
        if (tagged['statement']) {
            result.child_block.push(this.xml_to_json(tagged['statement'].lastChild, result));
        }
        return result;
    };
    DOMParserCompilerImpl.prototype.procedure_call_return_to_json = function (pre_block, child_nodes) {
        var named = child_nodes.named, tagged = child_nodes.tagged;
        var vars = tagged['mutation'];
        var result = pre_block;
        if (vars) {
            for (var i = 0; i < vars.children.length; i++) {
                var child = vars.children[i];
                result.params[child.getAttribute('name')] = result.params['ARG' + i];
                delete (result.params['ARG' + i]);
            }
        }
        result.procedure_name = vars.getAttribute('name');
        return result;
    };
    DOMParserCompilerImpl.prototype.procedure_call_no_return_to_json = function (pre_block, child_nodes) {
        var named = child_nodes.named, tagged = child_nodes.tagged;
        var vars = tagged['mutation'];
        var procedure_name = pre_block.params['NAME'];
        if (!_isString(procedure_name)) {
            throw this.ohno.compiler.system.procedure_call_name_not_string({ pre_block: pre_block });
        }
        var result = pre_block;
        result.procedure_name = procedure_name;
        if (vars) {
            for (var i = 0; i < vars.children.length; i++) {
                var child = vars.children[i];
                result.params[child.getAttribute('name')] = result.params['ARG' + i];
                delete (result.params['ARG' + i]);
                delete (result.params['ARGNAME' + i]);
            }
        }
        delete (result.params['WITH']);
        delete (result.params['NAME']);
        return result;
    };
    DOMParserCompilerImpl = tslib_1.__decorate([
        inversify_1.injectable(),
        tslib_1.__param(0, inversify_1.inject(di_symbols_1.BINDING.BlockPool)),
        tslib_1.__param(1, inversify_1.inject(di_symbols_1.BINDING.DOMParser)),
        tslib_1.__param(2, inversify_1.inject(di_symbols_1.BINDING.Ohno)),
        tslib_1.__param(3, inversify_1.inject(di_symbols_1.BINDING.BlockUtil)),
        tslib_1.__param(4, inversify_1.inject(di_symbols_1.BINDING.BlockRegistry)),
        tslib_1.__metadata("design:paramtypes", [Object, Object, Object, Object, Object])
    ], DOMParserCompilerImpl);
    return DOMParserCompilerImpl;
}());
exports.DOMParserCompilerImpl = DOMParserCompilerImpl;
