"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var nbus = require("nekobasu");
var custom = require("./custom");
function create_event_bus() {
    var event_bus = nbus.event_bus.categorized({
        error: nbus.event_bus.create({
            runtime: nbus.builtin.event(),
            all: nbus.builtin.event(),
        }),
        warning: nbus.event_bus.create({
            runtime: nbus.builtin.event(),
            all: nbus.builtin.event(),
        }),
        runtime_manager: nbus.event_bus.create({
            after_update: nbus.builtin.signal(),
            before_update: nbus.builtin.signal(),
            restart: nbus.builtin.signal(),
            stop: nbus.builtin.signal(),
            start: nbus.builtin.signal(),
            idle: nbus.builtin.signal(),
        }),
        runtime_data: nbus.event_bus.create({
            variable_update: custom.create_variable_neko(),
            list_update: custom.create_list_neko(),
            entity_variable_update: custom.create_entity_variable_neko(),
            entity_list_update: custom.create_entity_list_neko(),
            entity_dispose: nbus.builtin.event(),
            block_run_result: nbus.builtin.event(),
            test_done: nbus.builtin.signal(),
            block_running: nbus.builtin.event(),
            block_finished: nbus.builtin.event(),
        }),
        system: nbus.event_bus.create({
            config_updated: nbus.builtin.signal(),
        }),
        clones: nbus.event_bus.create({
            dispose_clone: nbus.builtin.event(),
        }),
    });
    bind_flush_events(event_bus);
    bind_collectors(event_bus);
    return event_bus;
}
exports.create_event_bus = create_event_bus;
function bind_collectors(eb) {
    eb.error.runtime.immediate.sub(function (e) {
        eb.error.all.send({ error: e.error });
    });
    eb.warning.runtime.immediate.sub(function (e) {
        eb.warning.all.send({ error: e.error });
    });
}
function bind_flush_events(eb) {
    eb.runtime_manager.after_update.immediate.sub(function () {
        eb.error._meta.flush();
        eb.warning._meta.flush();
        eb.runtime_manager._meta.flush();
        eb.runtime_data._meta.flush();
        eb.system._meta.flush();
        eb.clones._meta.flush();
    });
}
function as_public_event_bus(eb) {
    var ev_pub = nbus.util.builtin_event_as_public;
    var s_pub = nbus.util.builtin_signal_as_public;
    return {
        error: {
            runtime: ev_pub(eb.error.runtime),
            all: ev_pub(eb.error.all),
        },
        warning: {
            runtime: ev_pub(eb.warning.runtime),
            all: ev_pub(eb.warning.all),
        },
        runtime_manager: {
            after_update: s_pub(eb.runtime_manager.after_update),
            before_update: s_pub(eb.runtime_manager.before_update),
            restart: s_pub(eb.runtime_manager.restart),
            stop: s_pub(eb.runtime_manager.stop),
            start: s_pub(eb.runtime_manager.start),
            idle: s_pub(eb.runtime_manager.idle),
        },
        runtime_data: {
            variable_update: {
                merged: eb.runtime_data.variable_update.merged,
                last: eb.runtime_data.variable_update.last,
                list: eb.runtime_data.variable_update.list,
                immediate: eb.runtime_data.variable_update.immediate,
            },
            list_update: {
                merged: eb.runtime_data.list_update.merged,
                last: eb.runtime_data.list_update.last,
                list: eb.runtime_data.list_update.list,
                immediate: eb.runtime_data.list_update.immediate,
            },
            entity_variable_update: {
                merged: eb.runtime_data.entity_variable_update.merged,
                last: eb.runtime_data.entity_variable_update.last,
                list: eb.runtime_data.entity_variable_update.list,
                immediate: eb.runtime_data.entity_variable_update.immediate,
            },
            entity_list_update: {
                merged: eb.runtime_data.entity_list_update.merged,
                last: eb.runtime_data.entity_list_update.last,
                list: eb.runtime_data.entity_list_update.list,
                immediate: eb.runtime_data.entity_list_update.immediate,
            },
            entity_dispose: ev_pub(eb.runtime_data.entity_dispose),
            block_running: ev_pub(eb.runtime_data.block_running),
            block_finished: ev_pub(eb.runtime_data.block_finished),
            block_run_result: ev_pub(eb.runtime_data.block_run_result),
            test_done: s_pub(eb.runtime_data.test_done),
        },
    };
}
exports.as_public_event_bus = as_public_event_bus;
exports.DO_NOT_USE__TYPE_INFERENCE_HACK__EB = create_event_bus();
exports.DO_NOT_USE__TYPE_INFERENCE_HACK__EB_PUB = as_public_event_bus(create_event_bus());
