"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var r = require("resul-ts");
var _isString = require("lodash/isString");
var _last = require("lodash/last");
var _unescape = require("lodash/unescape");
var catastrophic_1 = require("catastrophic");
var inversify_1 = require("inversify");
var B = require("./block_types");
var block_types_1 = require("./block_types");
var di_symbols_1 = require("./di_symbols");
var HtmlParserCompilerImpl = (function () {
    function HtmlParserCompilerImpl(block_pool, html_parser, ohno, block, block_registry) {
        this.block_pool = block_pool;
        this.html_parser = html_parser;
        this.ohno = ohno;
        this.block = block;
        this.block_registry = block_registry;
    }
    HtmlParserCompilerImpl.prototype.is_hat_block = function (block_type) {
        return block_types_1.HAT_BLOCKS[block_type]
            || this.block.is.responder_type(block_type)
            || this.block_registry.block_restart_when_finished(block_type);
    };
    HtmlParserCompilerImpl.prototype.compile_entity = function (entity, force_compile_block_ids) {
        var xml = entity.blocksXML.trim();
        var w_json = this.compile_from_workspace_xml(xml, force_compile_block_ids);
        return tslib_1.__assign({}, entity, { procedures: w_json.procedures, compiled_block_map: w_json.compiled_block_map });
    };
    HtmlParserCompilerImpl.prototype.compile = function (entities, force_compile_block_ids, compile_all_blocks) {
        var _this = this;
        try {
            return r.success(entities.map(function (e) { return _this.compile_entity(e, force_compile_block_ids); }));
        }
        catch (e) {
            var err_msg = 'Heart could not compile one or more entities';
            if (e instanceof catastrophic_1.Catastrophe) {
                return r.error(err_msg, e);
            }
            else {
                return r.error(err_msg, this.ohno.compiler.system.unknown_compiler_error(e));
            }
        }
    };
    HtmlParserCompilerImpl.prototype.compile_from_workspace_xml = function (xml_string, force_compile_block_ids) {
        var _this = this;
        var res = {
            procedures: {},
            compiled_block_map: {},
        };
        if (!xml_string) {
            return res;
        }
        var handler = new this.html_parser.DomHandler(function (error, dom_list) {
            if (error) {
                return r.error('htmlparser2 compile error', error);
            }
            dom_list.forEach(function (dom) {
                var j = _this.xml_to_json(dom);
                if (j === undefined) {
                    return;
                }
                if (force_compile_block_ids) {
                    if (!force_compile_block_ids[j.id]) {
                        return;
                    }
                }
                else {
                    if (!_this.is_hat_block(j.type)) {
                        return;
                    }
                }
                if (_this.block.is.procedures_defnoreturn(j)) {
                    res.procedures[j.procedure_name] = j;
                }
                else {
                    res.compiled_block_map[j.id] = j;
                }
            });
        });
        var parser = new this.html_parser.Parser(handler);
        parser.write(xml_string);
        parser.end();
        return res;
    };
    HtmlParserCompilerImpl.prototype.parse_xml_field = function (xml_dom) {
        var innerHtml = (xml_dom.children.length) ? xml_dom.children[0].data : '';
        if (xml_dom.attribs.name == 'NUM') {
            return parseFloat(innerHtml);
        }
        return _unescape(innerHtml);
    };
    HtmlParserCompilerImpl.prototype.xml_to_json = function (xml_dom, parent) {
        var _this = this;
        if (!xml_dom) {
            return undefined;
        }
        var child_nodes = {
            named: {},
            tagged: {},
        };
        xml_dom.children.forEach(function (child) {
            var tag_name = child.name;
            child_nodes.tagged[tag_name] = child;
            var name = child.attribs.name;
            if (name) {
                child_nodes.named[name] = child;
            }
        });
        var next_block = child_nodes.tagged['next'];
        var next_block_json = next_block ? this.xml_to_json(next_block.children[0], parent) : undefined;
        var pre_block = this.block_pool.get();
        pre_block.params = {};
        pre_block.kind = xml_dom.attribs.type;
        pre_block.type = xml_dom.attribs.type;
        pre_block.disabled = !!xml_dom.attribs.disabled;
        pre_block.id = xml_dom.attribs.id;
        pre_block.parent_block = parent;
        pre_block.next_block = next_block_json;
        pre_block.child_block = [];
        pre_block.first_evaluation = true;
        pre_block.done_evaluating = false;
        pre_block.output_type = (xml_dom.parent && xml_dom.parent.name == 'value') ? B.BlockOutputType.number : B.BlockOutputType.none;
        xml_dom.children.forEach(function (child) {
            var child_name = child.attribs.name;
            if (child.name == 'value') {
                var param = _this.xml_to_json(_last(child.children), pre_block);
                if (param != undefined) {
                    pre_block.params[child_name] = param;
                }
            }
            else if (child.name == 'field') {
                pre_block.params[child_name] = _this.parse_xml_field(child);
            }
        });
        if (this.block.is.loop_block(pre_block)) {
            return this.loop_to_json(pre_block, child_nodes);
        }
        else if (this.block.is.cond_block(pre_block)) {
            return this.conditional_to_json(pre_block, child_nodes);
        }
        else if (this.block.is.event_block(pre_block)) {
            return this.event_to_json(pre_block, child_nodes);
        }
        else if (this.block.is.responder_block(pre_block)) {
            return this.responder_to_json(pre_block, child_nodes);
        }
        else if (this.block.is.proc_block(pre_block)) {
            return this.procedure_to_json(pre_block, child_nodes);
        }
        else if (this.block.is.async_tell(pre_block) || this.block.is.sync_tell(pre_block)) {
            return this.tell_to_json(pre_block, child_nodes);
        }
        else if (this.block.is.warp(pre_block)) {
            return this.warp_to_json(pre_block, child_nodes);
        }
        pre_block.kind = 'domain_block';
        return pre_block;
    };
    HtmlParserCompilerImpl.prototype.responder_to_json = function (pre_block, child_nodes) {
        pre_block.kind = 'action_block';
        var result = pre_block;
        if (child_nodes.tagged['statement']) {
            var statement_child = _last(child_nodes.tagged['statement'].children);
            result.child_block.push(this.xml_to_json(statement_child, result));
        }
        return result;
    };
    HtmlParserCompilerImpl.prototype.event_to_json = function (pre_block, child_nodes) {
        pre_block.kind = 'event_block';
        var result = pre_block;
        if (child_nodes.tagged['statement']) {
            var statement_child = _last(child_nodes.tagged['statement'].children);
            result.child_block.push(this.xml_to_json(statement_child, result));
        }
        return result;
    };
    HtmlParserCompilerImpl.prototype.loop_to_json = function (pre_block, child_nodes) {
        var named = child_nodes.named, tagged = child_nodes.tagged;
        var result = pre_block;
        if (this.block.is.repeat_n_times(result)
            || this.block.is.repeat_forever(result)
            || this.block.is.repeat_forever_until(result)) {
            result.child_block.push(named['DO'] ? this.xml_to_json(named['DO'].children[0], result) : undefined);
        }
        return result;
    };
    HtmlParserCompilerImpl.prototype.tell_to_json = function (pre_block, child_nodes) {
        var named = child_nodes.named, tagged = child_nodes.tagged;
        var result = pre_block;
        result.child_block.push(named['DO'] ? this.xml_to_json(named['DO'].children[0], result) : undefined);
        return result;
    };
    HtmlParserCompilerImpl.prototype.warp_to_json = function (pre_block, child_nodes) {
        var named = child_nodes.named, tagged = child_nodes.tagged;
        var result = pre_block;
        result.child_block.push(named['DO'] ? this.xml_to_json(named['DO'].children[0], result) : undefined);
        return result;
    };
    HtmlParserCompilerImpl.prototype.conditional_to_json = function (pre_block, child_nodes) {
        var named = child_nodes.named, tagged = child_nodes.tagged;
        var result = pre_block;
        result.params = {};
        result.conditions = [];
        function get_n_statements() {
            var keys = Object.keys(named);
            var max_if = 0;
            var max_do = 0;
            keys.forEach(function (key) {
                if (key.indexOf('IF') >= 0) {
                    max_if = Math.max(max_if, parseInt(key.split('IF')[1]));
                }
                else if (key.indexOf('DO') >= 0) {
                    max_do = Math.max(max_do, parseInt(key.split('DO')[1]));
                }
            });
            return Math.max(max_if, max_do) + 1;
        }
        var n_statements = get_n_statements();
        for (var i = 0; i < n_statements; ++i) {
            var condition = named['IF' + i];
            if (condition) {
                result.conditions.push(this.xml_to_json(_last(condition.children), pre_block));
            }
            else {
                result.conditions.push(undefined);
            }
            var statement = named['DO' + i];
            if (statement) {
                result.child_block.push(this.xml_to_json(_last(statement.children), pre_block));
            }
            else {
                result.child_block.push(undefined);
            }
        }
        var else_statement = named['ELSE'];
        if (else_statement) {
            result.child_block.push(this.xml_to_json(_last(else_statement.children), pre_block));
        }
        else {
            result.child_block.push(undefined);
        }
        return result;
    };
    HtmlParserCompilerImpl.prototype.procedure_to_json = function (pre_block, child_nodes) {
        switch (B.PROCEDURE_BLOCKS[pre_block.type]) {
            case B.PROCEDURE_BLOCKS.procedures_defnoreturn: return this.procedure_definition_to_json(pre_block, child_nodes);
            case B.PROCEDURE_BLOCKS.procedures_callreturn: return this.procedure_call_return_to_json(pre_block, child_nodes);
            case B.PROCEDURE_BLOCKS.procedures_callnoreturn: return this.procedure_call_no_return_to_json(pre_block, child_nodes);
            case B.PROCEDURE_BLOCKS.procedures_return_value: return pre_block;
            case B.PROCEDURE_BLOCKS.procedures_parameter: return pre_block;
        }
        throw this.ohno.compiler.system.unknown_procedure_block_type({ pre_block: pre_block, child_nodes: child_nodes });
    };
    HtmlParserCompilerImpl.prototype.procedure_definition_to_json = function (pre_block, child_nodes) {
        var named = child_nodes.named, tagged = child_nodes.tagged;
        var procedure_name = pre_block.params['NAME'];
        if (!_isString(procedure_name)) {
            throw this.ohno.compiler.system.procedure_name_not_string({ pre_block: pre_block });
        }
        var result = pre_block;
        result.procedure_name = procedure_name;
        result.params = {};
        var vars = tagged['mutation'];
        if (vars) {
            vars.children.forEach(function (var_dom) {
                result.params[var_dom.attribs.name] = true;
            });
        }
        if (tagged['statement']) {
            result.child_block.push(this.xml_to_json(_last(tagged['statement'].children), result));
        }
        return result;
    };
    HtmlParserCompilerImpl.prototype.procedure_call_return_to_json = function (pre_block, child_nodes) {
        var named = child_nodes.named, tagged = child_nodes.tagged;
        var vars = tagged['mutation'];
        var result = pre_block;
        if (vars) {
            vars.children.forEach(function (child, i) {
                result.params[child.attribs.name] = result.params['ARG' + i];
                delete (result.params['ARG' + i]);
            });
        }
        result.procedure_name = vars.attribs.name;
        return result;
    };
    HtmlParserCompilerImpl.prototype.procedure_call_no_return_to_json = function (pre_block, child_nodes) {
        var named = child_nodes.named, tagged = child_nodes.tagged;
        var vars = tagged['mutation'];
        var procedure_name = pre_block.params['NAME'];
        if (!_isString(procedure_name)) {
            throw this.ohno.compiler.system.procedure_call_name_not_string({ pre_block: pre_block });
        }
        var result = pre_block;
        result.procedure_name = procedure_name;
        if (vars) {
            vars.children.forEach(function (child, i) {
                result.params[child.attribs.name] = result.params['ARG' + i];
                delete (result.params['ARG' + i]);
                delete (result.params['ARGNAME' + i]);
            });
        }
        delete (result.params['WITH']);
        delete (result.params['NAME']);
        return result;
    };
    HtmlParserCompilerImpl = tslib_1.__decorate([
        inversify_1.injectable(),
        tslib_1.__param(0, inversify_1.inject(di_symbols_1.BINDING.BlockPool)),
        tslib_1.__param(1, inversify_1.inject(di_symbols_1.BINDING.HtmlParser)),
        tslib_1.__param(2, inversify_1.inject(di_symbols_1.BINDING.Ohno)),
        tslib_1.__param(3, inversify_1.inject(di_symbols_1.BINDING.BlockUtil)),
        tslib_1.__param(4, inversify_1.inject(di_symbols_1.BINDING.BlockRegistry)),
        tslib_1.__metadata("design:paramtypes", [Object, Object, Object, Object, Object])
    ], HtmlParserCompilerImpl);
    return HtmlParserCompilerImpl;
}());
exports.HtmlParserCompilerImpl = HtmlParserCompilerImpl;
