import * as r from 'resul-ts';
import { Catastrophe } from 'catastrophic';
import { DOMParser } from './dom_parser';
import * as H from './di_interfaces';
import * as P from './public_interfaces';
import * as BP from './block_provider';
import * as block_provider from './block_provider';
import * as toolbox from './toolbox';
import * as event_types from './event/public_types';
import * as basic_types from './basic_types';
import * as block_types from './block_types';
import * as interfaces from './public_interfaces';
import { Blockly } from './blockly_interface';
import * as dev_tool from './dev_tool';
export { Blockly, Catastrophe, basic_types, block_provider, block_types, dev_tool, event_types, interfaces, toolbox };
export declare type HeartConfig = H.PartialHeartConfig;
export interface HeartSpec1 {
    version: 1;
    logger?: basic_types.Logger;
    configuration?: H.PartialHeartConfig;
    compiler_requirements?: {
        dom_parser: DOMParser;
    } | {
        html_parser: any;
    };
    workspace_requirements?: {
        blockly: Blockly;
    };
    basic_blocks_requirements?: {
        day_names: basic_types.DayNames;
    };
}
export declare type HeartSpec = HeartSpec1;
export declare type MaybeCompiler = r.Result<P.Compiler, void>;
export declare type MaybeBlockXMLBuilder = r.Result<P.BlockXMLBuilder, void>;
export declare class Heart {
    private spec;
    private container;
    private block_registry?;
    private compiler?;
    private event_bus?;
    private runtime_data?;
    private runtime_manager?;
    private util?;
    private assertion_tool?;
    private block_xml_builder_factory?;
    private toolbox_factory?;
    private basic_block_provider_factory?;
    private basic_block_getters?;
    private test_block_provider_factory?;
    private test_block_getters?;
    private benchmark_block_provider_factory?;
    private benchmark_block_getters?;
    constructor(spec: HeartSpec1);
    get_event_bus(): P.EventBus;
    get_compiler(): MaybeCompiler;
    get_util(): P.Util;
    get_runtime_data(): P.RuntimeData;
    get_runtime_manager(): P.RuntimeManager;
    get_block_registry(): P.BlockRegistry;
    get_block_xml_builder(_block_provider: BP.BlockProvider): MaybeBlockXMLBuilder;
    get_toolbox(toolbox_config: toolbox.ToolboxConfig, _block_xml: P.BlockXML): P.Toolbox;
    get_assertion_tool(): P.TestAssertionTool;
    basic_blocks(): {
        load_runtime_provider: () => void;
        init: (basic_block_config_deps: P.BasicBlockConfigDependencies) => {
            get_block_provider: () => P.BlockProvider;
            get_block_xml: () => r.Result<H.BlockXML, void>;
        };
    };
    test_blocks(ava_like_test_object: P.AvaTestLike): {
        load_runtime_provider: () => void;
        init: (test_icon_url: string) => {
            get_block_provider: () => P.BlockProvider;
            get_block_xml: () => r.Result<H.BlockXML, void>;
        };
    };
    benchmark_blocks(benchmark_tool: P.BenchmarkTool): {
        load_runtime_provider: () => void;
        init: (benchmark_icon_url: string) => {
            get_block_provider: () => P.BlockProvider;
            get_block_xml: () => r.Result<H.BlockXML, void>;
        };
    };
    create_domain_function_error(error_properties: basic_types.ClientErrorProperties): Catastrophe;
    set_config(config: H.PartialHeartConfig): void;
}
export declare function new_heart(spec: HeartSpec): Heart;
