"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var r = require("resul-ts");
var catastrophic_1 = require("catastrophic");
exports.Catastrophe = catastrophic_1.Catastrophe;
var event_bus_1 = require("./event/event_bus");
var container_1 = require("./container");
var di_symbols_1 = require("./di_symbols");
var block_provider = require("./block_provider");
exports.block_provider = block_provider;
var toolbox = require("./toolbox");
exports.toolbox = toolbox;
var event_types = require("./event/public_types");
exports.event_types = event_types;
var basic_types = require("./basic_types");
exports.basic_types = basic_types;
var block_types = require("./block_types");
exports.block_types = block_types;
var interfaces = require("./public_interfaces");
exports.interfaces = interfaces;
var dev_tool = require("./dev_tool");
exports.dev_tool = dev_tool;
var Heart = (function () {
    function Heart(spec) {
        this.spec = spec;
        var basic_blocks_requirements = spec.basic_blocks_requirements, compiler_requirements = spec.compiler_requirements, configuration = spec.configuration, logger = spec.logger, workspace_requirements = spec.workspace_requirements;
        var logger_ = logger;
        if (logger_ == undefined) {
            var cwarn = console.warn.bind(console);
            var cerr = console.error.bind(console);
            var clog = console.log.bind(console);
            logger_ = {
                fatal: cerr,
                error: cerr,
                warn: cwarn,
                info: clog,
                debug: clog,
                trace: clog,
            };
        }
        this.container = container_1.get_instance({
            configuration: configuration || {},
            basic_blocks_requirements: basic_blocks_requirements,
            compiler_requirements: compiler_requirements,
            logger: logger_,
            workspace_requirements: workspace_requirements,
        });
    }
    Heart.prototype.get_event_bus = function () {
        if (this.event_bus == undefined) {
            this.event_bus = event_bus_1.as_public_event_bus(this.container.get(di_symbols_1.BINDING.EventBus));
        }
        return this.event_bus;
    };
    Heart.prototype.get_compiler = function () {
        if (this.compiler == undefined) {
            if (this.spec.compiler_requirements == undefined) {
                return r.fail('Heart compiler requirements unfulfilled, cannot get compiler');
            }
            this.compiler = this.container.get(di_symbols_1.BINDING.Compiler);
        }
        return r.success(this.compiler);
    };
    Heart.prototype.get_util = function () {
        if (this.util == undefined) {
            this.util = this.container.get(di_symbols_1.BINDING.Util);
        }
        return this.util;
    };
    Heart.prototype.get_runtime_data = function () {
        if (this.runtime_data == undefined) {
            this.runtime_data = this.container.get(di_symbols_1.BINDING.RuntimeData);
        }
        return this.runtime_data;
    };
    Heart.prototype.get_runtime_manager = function () {
        if (this.runtime_manager == undefined) {
            this.runtime_manager = this.container.get(di_symbols_1.BINDING.RuntimeManager);
        }
        return this.runtime_manager;
    };
    Heart.prototype.get_block_registry = function () {
        if (this.block_registry == undefined) {
            this.block_registry = this.container.get(di_symbols_1.BINDING.BlockRegistry);
        }
        return this.block_registry;
    };
    Heart.prototype.get_block_xml_builder = function (_block_provider) {
        if (this.block_xml_builder_factory == undefined) {
            if (this.spec.workspace_requirements == undefined) {
                return r.fail('Heart workspace requirements unfulfilled, cannot get xml builder factory.');
            }
            this.block_xml_builder_factory = this.container.get(di_symbols_1.BINDING.BlockXmlBuilderFactory);
        }
        return r.success(this.block_xml_builder_factory.create(_block_provider));
    };
    Heart.prototype.get_toolbox = function (toolbox_config, _block_xml) {
        if (this.toolbox_factory == undefined) {
            this.toolbox_factory = this.container.get(di_symbols_1.BINDING.ToolboxFactory);
        }
        return this.toolbox_factory.create(toolbox_config, _block_xml);
    };
    Heart.prototype.get_assertion_tool = function () {
        if (this.assertion_tool == undefined) {
            this.assertion_tool = this.container.get(di_symbols_1.BINDING.TestAssertionTool);
        }
        return this.assertion_tool;
    };
    Heart.prototype.basic_blocks = function () {
        var _this = this;
        if (this.basic_block_provider_factory == undefined) {
            this.basic_block_provider_factory =
                this.container.get(di_symbols_1.BINDING.BasicBlockProviderFactory);
        }
        var bbpf = this.basic_block_provider_factory;
        return {
            load_runtime_provider: function () {
                _this.get_block_registry().register_runtime_provider(bbpf.runtime_provider());
            },
            init: function (basic_block_config_deps) {
                if (_this.basic_block_getters == undefined) {
                    _this.basic_block_getters = bbpf.block_provider_and_xml(basic_block_config_deps);
                }
                var bbg = _this.basic_block_getters;
                return {
                    get_block_provider: function () {
                        return bbg.get_block_provider();
                    },
                    get_block_xml: function () {
                        return bbg.get_default_block_xml();
                    },
                };
            },
        };
    };
    Heart.prototype.test_blocks = function (ava_like_test_object) {
        var _this = this;
        if (this.test_block_provider_factory == undefined) {
            this.test_block_provider_factory =
                this.container.get(di_symbols_1.BINDING.TestBlockProviderFactory);
        }
        var bbpf = this.test_block_provider_factory;
        return {
            load_runtime_provider: function () {
                _this.get_block_registry().register_runtime_provider(bbpf.runtime_provider(ava_like_test_object));
            },
            init: function (test_icon_url) {
                if (_this.test_block_getters == undefined) {
                    _this.test_block_getters = bbpf.block_provider_and_xml(test_icon_url, ava_like_test_object);
                }
                var bbg = _this.test_block_getters;
                return {
                    get_block_provider: function () {
                        return bbg.get_block_provider();
                    },
                    get_block_xml: function () {
                        return bbg.get_default_block_xml();
                    },
                };
            },
        };
    };
    Heart.prototype.benchmark_blocks = function (benchmark_tool) {
        var _this = this;
        if (this.benchmark_block_provider_factory == undefined) {
            this.benchmark_block_provider_factory =
                this.container.get(di_symbols_1.BINDING.BenchmarkBlockProviderFactory);
        }
        var bbpf = this.benchmark_block_provider_factory;
        return {
            load_runtime_provider: function () {
                _this.get_block_registry().register_runtime_provider(bbpf.runtime_provider(benchmark_tool));
            },
            init: function (benchmark_icon_url) {
                if (_this.benchmark_block_getters == undefined) {
                    _this.benchmark_block_getters = bbpf.block_provider_and_xml(benchmark_icon_url, benchmark_tool);
                }
                var bbg = _this.benchmark_block_getters;
                return {
                    get_block_provider: function () {
                        return bbg.get_block_provider();
                    },
                    get_block_xml: function () {
                        return bbg.get_default_block_xml();
                    },
                };
            },
        };
    };
    Heart.prototype.create_domain_function_error = function (error_properties) {
        var util = this.container.get(di_symbols_1.BINDING.Util);
        var annotation = {
            client_annotation: error_properties,
        };
        if (error_properties.native_error != undefined) {
            return util.ohno.client.domain_function_error(error_properties.native_error, annotation);
        }
        return util.ohno.client.domain_function_error(annotation);
    };
    Heart.prototype.set_config = function (config) {
        this.container.get(di_symbols_1.BINDING.Config).set(config);
    };
    return Heart;
}());
exports.Heart = Heart;
function new_heart(spec) {
    return new Heart(spec);
}
exports.new_heart = new_heart;
