"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _cloneDeep = require("lodash/cloneDeep");
var _defaults = require("lodash/defaults");
var catastrophic_1 = require("catastrophic");
var T = require("../basic_types");
function peek(stack) {
    if (stack.length == 0) {
        return undefined;
    }
    return stack[stack.length - 1];
}
var MAIN_PROGRAM_ID = '__main__';
var OptiRunner = (function () {
    function OptiRunner(u, ohno, should_pretty_print, tell_should_ensure_entity_exists, should_report_current_running_block, max_procedure_calls_per_interpreter_step, max_warp_iterations_per_interpreter_step, run_mgr, block_pool, program_cache, domain_functions, original_identities, priorities, program, group_id, compiled_block, variable_specs, is_warped, max_call_stack_size, action_parameters, on_finished) {
        this.u = u;
        this.ohno = ohno;
        this.should_pretty_print = should_pretty_print;
        this.tell_should_ensure_entity_exists = tell_should_ensure_entity_exists;
        this.should_report_current_running_block = should_report_current_running_block;
        this.max_procedure_calls_per_interpreter_step = max_procedure_calls_per_interpreter_step;
        this.max_warp_iterations_per_interpreter_step = max_warp_iterations_per_interpreter_step;
        this.run_mgr = run_mgr;
        this.block_pool = block_pool;
        this.program_cache = program_cache;
        this.domain_functions = domain_functions;
        this.original_identities = original_identities;
        this.priorities = priorities;
        this.program = program;
        this.max_call_stack_size = max_call_stack_size;
        this.action_parameters = action_parameters;
        this.on_finished = on_finished;
        this.variables = {};
        this.variables_stack = [];
        this.programs = {};
        this.current_frame = undefined;
        this.n_warp_iterations = 0;
        this.metadata = {
            priorities: priorities,
            original_identities: original_identities,
            typeclass_id: original_identities.typeclass_id,
            interpreter_id: original_identities.interpreter_id,
            type: compiled_block.type,
            group_id: group_id,
        };
        this.root_block_type = compiled_block.type;
        this.root_block_id = original_identities.interpreter_id;
        for (var var_id in variable_specs) {
            this.variables[var_id] = variable_specs[var_id].value;
        }
        this.original_variables = _cloneDeep(this.variables);
        this.step_input_outputs = {
            ohno: this.ohno,
            call_domain_function: this.step_call_domain_function.bind(this),
            report_sync_telling: this.step_report_sync_telling.bind(this),
            async_tell: this.step_async_tell.bind(this),
        };
        this.step_outputs = {
            proc_call: this.step_call_procedure.bind(this),
            before_expression: this.step_before_expression.bind(this),
            after_potential_blocker: this.step_after_potential_blocker.bind(this),
            increment_program_counter: this.step_increment_program_counter.bind(this),
            reset_program_counter: this.step_reset_program_counter.bind(this),
            after_iteration: this.step_iteration_yield.bind(this),
            finished: this.step_finished.bind(this),
            proc_do_return_value: this.step_return_value.bind(this),
            proc_yield_after_call: this.step_yield_to_procedure.bind(this),
        };
        this.domain_function_util = {
            runtime_manager: this.run_mgr,
            add_user_procedure_call_to_stack: this.step_call_procedure.bind(this),
            get_action_parameter: this.get_action_parameter.bind(this),
        };
        this.programs[MAIN_PROGRAM_ID] = program;
        this.reset(is_warped);
    }
    OptiRunner.prototype.reset = function (is_warped) {
        this.program_state = this.empty_state(this.original_identities.target_entity, is_warped || false);
    };
    OptiRunner.prototype.get_action_parameter = function (parameter_id) {
        if (this.action_parameters == undefined) {
            return undefined;
        }
        return this.action_parameters[parameter_id];
    };
    OptiRunner.prototype.step_call_procedure = function (function_id, target_entity_id, parameters, proc_call_bid, is_inside_warp) {
        var procedure_container = this.run_mgr.get_procedure(function_id, false);
        if (procedure_container == undefined) {
            throw this.u.ohno.user.call_undefined_procedure({
                block_id: proc_call_bid,
            });
        }
        var proc_stack = this.program_state.proc_stack;
        console.log(proc_stack.length);
        if (proc_stack.length > this.max_call_stack_size) {
            throw this.ohno.system.stack_overflow({
                block_id: proc_call_bid,
                procedure_id: function_id,
            });
        }
        var procedure = this.programs[function_id];
        if (procedure == undefined) {
            procedure = this.program_cache.get_program(procedure_container.script.id, procedure_container.source_entity_id, procedure_container.script.id, this.original_identities.interpreter_id, procedure_container.script);
            this.programs[function_id] = procedure;
        }
        var frame = this.empty_stack_frame(function_id, target_entity_id, procedure, is_inside_warp || this.current_frame.is_warped, parameters, procedure_container.source_entity_id, procedure_container.script.id, proc_call_bid);
        this.program_state.proc_stack.push(frame);
        this.program_state.did_proc_yield = true;
    };
    OptiRunner.prototype.step_yield_to_procedure = function (yield_group_id, next_statement_id) {
        var stack = this.program_state.proc_stack;
        var prev_frame = stack[stack.length - 2];
        if (prev_frame != undefined) {
            prev_frame.program_counter[yield_group_id] = next_statement_id;
        }
    };
    OptiRunner.prototype.step_iteration_yield = function (is_inside_warp) {
        var is_warped = this.current_frame.is_warped || is_inside_warp;
        var should_yield = this.n_warp_iterations > this.max_warp_iterations_per_interpreter_step || !is_warped;
        if (should_yield == false) {
            this.n_warp_iterations++;
        }
        return should_yield;
    };
    OptiRunner.prototype.step_before_expression = function (block_id, source_map_entity, source_map_rbid, yield_group_id, next_statement_id) {
        if (this.should_report_current_running_block) {
            this.run_mgr.set_running_block(source_map_rbid, block_id);
        }
        this.current_frame.program_counter[yield_group_id] = next_statement_id;
        return false;
    };
    OptiRunner.prototype.step_after_potential_blocker = function (block_id, source_map_entity, source_map_rbid, yield_group_id, next_statement_id) {
        this.current_frame.program_counter[yield_group_id] = next_statement_id;
        var must_yield = this.run_mgr.current_interpreter_must_yield(this.root_block_id, this.metadata.group_id);
        return must_yield || this.program_state.did_proc_yield;
    };
    OptiRunner.prototype.step_increment_program_counter = function (source_map_entity, source_map_rbid, yield_group_id, next_statement_id) {
        this.current_frame.program_counter[yield_group_id] = next_statement_id;
    };
    OptiRunner.prototype.step_reset_program_counter = function (source_map_entity, source_map_rbid, yield_group_id) {
        this.current_frame.program_counter[yield_group_id] = 0;
    };
    OptiRunner.prototype.step_finished = function () {
        this.program_state.proc_stack.pop();
        this.current_frame = undefined;
        this.program_state.proc_has_return_value = false;
        this.program_state.proc_return_value = undefined;
        this.program_state.did_proc_yield = true;
        if (this.program_state.proc_stack.length == 0) {
            this.program_state.did_proc_yield = false;
            this.program_state.finished = true;
            return;
        }
    };
    OptiRunner.prototype.step_return_value = function (block_id, value) {
        if (value == undefined) {
            throw this.ohno.user.procedure_return_empty({
                block_id: block_id,
                root_block_id: this.current_frame.source_map_rbid,
                entity_id: this.current_frame.source_map_entity,
            });
        }
        var returning_frame = this.program_state.proc_stack.pop();
        this.current_frame = undefined;
        this.program_state.proc_has_return_value = true;
        this.program_state.proc_return_value = value;
        this.program_state.did_proc_yield = true;
    };
    OptiRunner.prototype.step_call_domain_function = function (return_value_storage_id, function_id, block_id, args, target_entity) {
        var _this = this;
        if (this.domain_functions[function_id] == undefined) {
            throw this.ohno.system.missing_domain_function({
                block_id: block_id,
                block_type: function_id,
                root_block_id: this.current_frame.source_map_rbid,
                entity_id: this.current_frame.source_map_entity,
            });
        }
        var res;
        try {
            res = this.domain_functions[function_id](args, this.original_identities.interpreter_id, target_entity, this.domain_function_util);
        }
        catch (e) {
            throw this.extend_domain_function_call_error(e, block_id, function_id, this.current_frame.source_map_rbid, this.current_frame.source_map_entity);
        }
        if (res == undefined) {
            return;
        }
        if (typeof res.then == 'function') {
            var lock_handle_1 = this.run_mgr.get_thread_lock(target_entity, this.original_identities.interpreter_id);
            var current_frame_1 = this.current_frame;
            var run_mgr_1 = this.run_mgr;
            var on_success = function (res_val) {
                current_frame_1.dynamic_data[return_value_storage_id] = res_val;
                lock_handle_1.stop();
            };
            var on_fail = function (e) {
                var error = _this.extend_domain_function_call_error(e, block_id, function_id, current_frame_1.source_map_rbid, current_frame_1.source_map_entity);
                run_mgr_1.report_error_and_stop(error, 'OptiRunner domain function promise result handler');
            };
            res.then(on_success, on_fail);
        }
        this.current_frame.dynamic_data[return_value_storage_id] = res;
    };
    OptiRunner.prototype.extend_domain_function_call_error = function (e, block_id, function_id, source_map_rbid, source_map_entity) {
        var metadata = {
            block_id: block_id,
            block_type: function_id,
        };
        if (e instanceof catastrophic_1.Catastrophe) {
            e.annotation = e.annotation || {};
            _defaults(e.annotation, metadata);
            return e;
        }
        return this.ohno.system.unknown_error_in_domain_function_call(e, metadata);
    };
    OptiRunner.prototype.ensure_tell_target_ok = function (block_id, target_entity) {
        var entity_state = this.run_mgr.get_entity_state(target_entity);
        if (this.tell_should_ensure_entity_exists) {
            if (entity_state == T.EntityState.Destructing) {
                this.run_mgr.report_warning(this.ohno.warning.tell_with_destructing_entity({
                    root_block_id: this.current_frame.source_map_rbid,
                    entity_id: this.current_frame.source_map_entity,
                    block_id: block_id,
                    tell_entity: target_entity,
                }));
                return false;
            }
            else if (entity_state == T.EntityState.Disposed) {
                this.run_mgr.report_warning(this.ohno.warning.tell_with_disposed_entity({
                    root_block_id: this.current_frame.source_map_rbid,
                    entity_id: this.current_frame.source_map_entity,
                    block_id: block_id,
                    tell_entity: target_entity,
                }));
                return false;
            }
            else if (entity_state == T.EntityState.Unknown) {
                throw this.ohno.user.tell_with_unknown_entity({
                    root_block_id: this.current_frame.source_map_rbid,
                    entity_id: this.current_frame.source_map_entity,
                    block_id: block_id,
                    tell_entity: target_entity,
                });
            }
        }
        return true;
    };
    OptiRunner.prototype.step_async_tell = function (block_id, target_entity, new_target_entity, is_inside_warp) {
        if (!this.ensure_tell_target_ok(block_id, target_entity)) {
            return;
        }
        this.run_mgr.spawn_async_tell_interpreter({
            typeclass_id: this.original_identities.typeclass_id,
            interpreter_id: this.original_identities.interpreter_id,
            target_entity: target_entity,
            source_map_entity: this.current_frame.source_map_entity,
            source_map_rbid: this.current_frame.source_map_rbid,
        }, new_target_entity, this.current_frame.async_tell_asts[block_id], this.metadata.group_id, is_inside_warp);
    };
    OptiRunner.prototype.step_report_sync_telling = function (block_id, target_entity) {
        this.ensure_tell_target_ok(block_id, target_entity);
    };
    OptiRunner.prototype.empty_stack_frame = function (id, target_entity_id, program, is_warped, proc_params, source_map_entity, source_map_rbid, proc_call_bid) {
        var n_yield_groups = program.n_yield_groups;
        var program_counter = [];
        for (var i = 0; i < n_yield_groups; i++) {
            program_counter.push(0);
        }
        return {
            proc_id: id,
            proc_parameters: proc_params || {},
            program_counter: program_counter,
            dynamic_data: this.should_pretty_print ? {} : new Array(program.dynamic_data_size),
            target_entity_id: target_entity_id,
            source_map_entity: source_map_entity || this.original_identities.source_map_entity,
            source_map_rbid: source_map_rbid || this.original_identities.source_map_rbid,
            async_tell_asts: program.async_tell_asts,
            proc_call_bid: proc_call_bid,
            is_warped: is_warped,
        };
    };
    OptiRunner.prototype.empty_state = function (target_entity_id, is_warped) {
        return {
            finished: false,
            proc_stack: [
                this.empty_stack_frame(MAIN_PROGRAM_ID, target_entity_id, this.program, is_warped),
            ],
            proc_has_return_value: false,
            proc_return_value: undefined,
            did_proc_yield: false,
        };
    };
    OptiRunner.prototype.get_step_args = function (frame, static_data) {
        var inputs = {
            identities: this.original_identities,
            default_target_entity_id: frame.target_entity_id,
            static_data: static_data,
            proc_parameters: frame.proc_parameters,
            proc_return_value: this.program_state.proc_return_value,
            proc_has_return_value: this.program_state.proc_has_return_value,
        };
        inputs.identities.source_map_entity = frame.source_map_entity;
        inputs.identities.source_map_rbid = frame.source_map_rbid;
        var state = {
            dynamic_data: frame.dynamic_data,
            program_counter: frame.program_counter,
        };
        var outputs = this.step_outputs;
        if (this.should_pretty_print) {
            return {
                ohno: this.step_input_outputs.ohno,
                call_domain_function: this.step_input_outputs.call_domain_function,
                report_sync_telling: this.step_input_outputs.report_sync_telling,
                async_tell: this.step_input_outputs.async_tell,
                default_target_entity_id: inputs.default_target_entity_id,
                identities: inputs.identities,
                static_data: inputs.static_data,
                proc_parameters: inputs.proc_parameters,
                proc_return_value: inputs.proc_return_value,
                proc_has_return_value: inputs.proc_has_return_value,
                proc_do_return_value: outputs.proc_do_return_value,
                proc_yield_after_call: outputs.proc_yield_after_call,
                dynamic_data: state.dynamic_data,
                program_counter: state.program_counter,
                proc_call: outputs.proc_call,
                before_expression: outputs.before_expression,
                after_potential_blocker: outputs.after_potential_blocker,
                increment_program_counter: outputs.increment_program_counter,
                reset_program_counter: outputs.reset_program_counter,
                after_iteration: outputs.after_iteration,
                finished: outputs.finished,
            };
        }
        return {
            AA: this.step_input_outputs.ohno,
            AB: this.step_input_outputs.call_domain_function,
            AC: this.step_input_outputs.report_sync_telling,
            AD: this.step_input_outputs.async_tell,
            BA: inputs.default_target_entity_id,
            BB: inputs.identities,
            BC: inputs.static_data,
            CA: inputs.proc_parameters,
            CB: inputs.proc_return_value,
            CC: inputs.proc_has_return_value,
            CD: outputs.proc_do_return_value,
            CE: outputs.proc_yield_after_call,
            DA: state.dynamic_data,
            DB: state.program_counter,
            EA: outputs.proc_call,
            EB: outputs.before_expression,
            EC: outputs.after_potential_blocker,
            ED: outputs.increment_program_counter,
            EE: outputs.reset_program_counter,
            EF: outputs.after_iteration,
            EG: outputs.finished,
        };
    };
    OptiRunner.prototype.step = function () {
        var n_procedure_calls = 0;
        while (true) {
            var step_res = this.do_step();
            if (step_res != undefined) {
                return step_res;
            }
            n_procedure_calls++;
            if (n_procedure_calls >= this.max_procedure_calls_per_interpreter_step) {
                return T.StepResult.yielding;
            }
        }
    };
    OptiRunner.prototype.do_step = function () {
        var frame = peek(this.program_state.proc_stack);
        if (frame == undefined) {
            throw this.ohno.system.procedure_popped_empty_call_stack({
                root_block_id: this.original_identities.source_map_rbid,
                entity_id: this.original_identities.source_map_entity,
            });
        }
        this.current_frame = frame;
        this.n_warp_iterations = 0;
        this.program_state.did_proc_yield = false;
        var program = this.programs[frame.proc_id];
        var args = this.get_step_args(frame, program.static_data);
        this.run_mgr.set_current_interpreter_not_blocked();
        try {
            program.script(args);
        }
        catch (e) {
            var metadata = {
                interpreter_id: this.original_identities.interpreter_id,
                interpreter_stack: this.get_current_stack(),
            };
            if (e instanceof catastrophic_1.Catastrophe) {
                e.annotation = e.annotation || {};
                _defaults(e.annotation, metadata);
                throw e;
            }
            throw this.ohno.system.unknown_system_error(e, metadata);
        }
        if (this.program_state.finished) {
            if (this.on_finished !== undefined) {
                this.on_finished(this.original_identities.source_map_rbid);
            }
            return T.StepResult.finished;
        }
        if (this.program_state.did_proc_yield == false) {
            return T.StepResult.yielding;
        }
        return undefined;
    };
    OptiRunner.prototype.get_current_stack = function () {
        var frame_stack = this.program_state.proc_stack;
        var stack = [];
        for (var i = 0; i < frame_stack.length; i++) {
            var current_frame = frame_stack[i];
            stack.push({
                interpreter_id: this.original_identities.interpreter_id,
                source_entity_id: current_frame.source_map_entity,
                source_map_rbid: current_frame.source_map_rbid,
                block_id: (i === frame_stack.length - 1) ? undefined : frame_stack[i + 1].proc_call_bid,
                proc_parameters: _cloneDeep(current_frame.proc_parameters),
                proc_id: (current_frame.proc_id === MAIN_PROGRAM_ID) ? undefined : current_frame.proc_id,
            });
        }
        return stack;
    };
    OptiRunner.prototype.dispose = function () {
    };
    OptiRunner.prototype.set_variable = function (name, value) {
        this.variables[name] = value;
    };
    OptiRunner.prototype.get_variable = function (name) {
        return this.variables[name];
    };
    OptiRunner.prototype.get_variables = function () {
        return this.variables;
    };
    return OptiRunner;
}());
exports.OptiRunner = OptiRunner;
