import * as rlt from 'resul-ts';
import { Catastrophe } from 'catastrophic';
import { InitRunnableResult, List } from './public_interfaces';
import { CompiledEntity, Dict, EntityState, ID, Identities, MilliSeconds, ProcedureContainer, Task, TaskHandle, VariableSpec } from './basic_types';
import { CompiledBlock, ProcedureDefinitionBlock } from './block_types';
import * as H from './di_interfaces';
import { Action } from './block_provider';
export interface Runnable {
    identities: Identities;
    compile_cache_id?: ID;
    script: CompiledBlock;
    on_finished?: H.OnInterpreterFinished;
    action_parameters?: Dict<any>;
    responder_priority?: number;
    is_warped?: boolean;
    group_id: ID | undefined;
}
export interface CompiledBlockMap {
    [rbid: string]: CompiledBlock;
}
export interface BlockEventMap {
    [rbid: string]: boolean;
}
export interface EntityToAdd {
    typeclass_id: ID;
    entity_id: ID;
    compiled_block_map: CompiledBlockMap;
    source_map_entity: ID;
    source_rbids: {
        [compiled_block_map_rbid: string]: string;
    };
    running_group_id: {
        [compiled_block_map_rbid: string]: ID;
    };
}
export declare class RuntimeManagerImpl implements H.RuntimeManager {
    private state_store;
    private bif;
    private orf;
    private block_pool;
    private block_registry;
    private event_bus;
    private log;
    private ohno;
    private prng_factory;
    private runtime_data;
    private task_manager;
    private u;
    private config;
    constructor(state_store: H.ActionStateStore, bif: H.BlockInterpreterFactory, orf: H.BlockInterpreterFactory, block_pool: H.BlockPool, block_registry: H.BlockRegistry, event_bus: H.EventBusPrivate, log: H.Logger, ohno: H.Ohno, prng_factory: H.PRNGFactory, runtime_data: H.RuntimeData, task_manager: H.TaskManager, u: H.Util, config: H.Config);
    private user_debug_mode;
    private deterministic;
    private entity_max_clones_per_frame;
    private prng;
    private frames;
    private interpreters_spawned;
    private timer_block_date;
    private interpreter_factory;
    private entity_states;
    private interpreters;
    private sorted_interpreters;
    private interpreter_sorters;
    private running_interpreter_was_blocked;
    private interpreters_to_restart_when_finished;
    private action_responders;
    private state_responders;
    private dynamic_responders;
    private action_queue;
    private action_parameters;
    private typeclasses;
    private procedure_compiled_block_map;
    private variable_specs;
    private script_variable_specs;
    private variables;
    private entity_variables;
    private entity_id_to_interpreter_id_dict;
    private interpreter_id_to_entity_id;
    private entity_id_to_compiled_blocks;
    private entity_id_to_typeclass_id;
    private running_constructor_to_typeclass;
    private running_destructor_to_typeclass;
    private tell_source_map;
    private responder_id_to_instance_interp_id;
    private instance_interp_id_to_responder_id;
    private just_restart;
    private just_stopped;
    private interpreters_needing_dispose;
    private entities_needing_add;
    private interpreters_needing_spawn;
    private entities_needing_destruct;
    private entities_needing_dispose;
    private entities_cloned_times;
    private running_group?;
    private running_group_changed;
    set_variable_specs(variable_specs: VariableSpec[]): void;
    change_running_group(id: string): void;
    interpreter_out_of_running_group(group_id?: string): boolean;
    block_finish_out_of_run_group(ns_id: string): boolean;
    clear(): void;
    private do_spawn_interpreter(r);
    private spawn_interpreter(r);
    private get_responder_filter_value(param);
    load(ces: CompiledEntity[]): rlt.Res[];
    private load_typeclass(ce);
    create_singleton_entity_instances(): InitRunnableResult[];
    create_entity_instance(typeclass_id: ID, entity_id: ID, params?: Dict<any>): InitRunnableResult[];
    private initialize_entity_runnables(typeclass_id, entity_id);
    private specialize_runnable(r, target_entity, action_parameters?);
    initialize_runnable(r: Runnable, target_entity?: ID, is_constructor?: boolean, is_destructor?: boolean): InitRunnableResult;
    send_action(action: Action): void;
    private runnable_should_respond_to_action(r, action);
    private create_action_responder_interpreters(actions);
    private create_state_responder_interpreters();
    private is_stateful_action(action_id);
    private create_dynamic_responder_interpreters(actions);
    private do_add_responder_interpreters(r, type, action_parameters?);
    private update_actions();
    private do_add_entity(e);
    add_entity(e: EntityToAdd): void;
    private get_cloned_entity_data(entity_id, is_mirror);
    clone_entity(entity_id: ID, is_mirror: boolean): ID | undefined;
    run(): void;
    delete_other_interpreters(interpreter_id: ID): void;
    stop(): void;
    restart(): void;
    private sort_interpreters();
    update(): void;
    set_current_interpreter_not_blocked(): void;
    current_interpreter_must_yield(interpreter_id: ID, group_id?: ID): boolean;
    add_task(t: Task): TaskHandle;
    get_thread_lock(entity_id: ID, interpreter_id: ID): TaskHandle;
    thread_wait(entity_id: ID, interpreter_id: ID, lifetime: MilliSeconds): void;
    procedure_load(source_entity_id: ID, procedure_name: string, compiled_block: ProcedureDefinitionBlock): void;
    get_procedure(p_name: string, do_clone?: boolean): ProcedureContainer | undefined;
    get_elapsed_frames(): number;
    reset_timer(): void;
    get_timer_elapsed_s(): number;
    private do_dispose_of_interpreter(interp_id, force_dispose?);
    private do_dispose_of_entity(entity_id);
    private do_destruct_entity(entity_id);
    private update_dispose();
    private get_interpreter_sorters(is_deterministic);
    private step_code();
    is_blocking(interpreter_id: ID): boolean;
    private _step(interpreter_id, interpreter);
    dispose_sprite(entity_id: string): void;
    destruct_entity(entity_id: string): void;
    dispose_block_group(interpreter_id: ID): void;
    dispose_all(): void;
    dispose_other_block_groups_of_entity(entity_id: ID, interpreter_to_keep_id: ID): void;
    dispose_block_groups_of_other_entities(entity_id: ID): void;
    private get_var_or_list(var_id, default_value, interpreter_id?, entity_id?);
    get_list_id(list: List, entity_id: ID): string | undefined;
    set_variable(var_id: string, val: any, interpreter_id?: ID, entity_id?: ID): void;
    get_variable(var_id: string, interpreter_id?: ID, entity_id?: ID): any;
    lists_get(var_id: string, interpreter_id?: ID, entity_id?: ID): any[];
    get_global_variable(var_id: string): any;
    is_entity_variable(var_id: string): boolean;
    get_entity_id_from_root_block_id(interp_id: ID): ID;
    spawn_async_tell_interpreter(teller_identities: Identities, new_target_entity: ID, script: CompiledBlock, group_id: ID | undefined, is_warped?: boolean): void;
    spawn_sync_tell_interpreter(teller_identities: Identities, new_target_entity: ID, script: CompiledBlock, group_id: ID | undefined): void;
    private do_spawn_tell_interpreter(teller_identities, new_target_entity, script, group_id, on_finished?, is_warped?);
    private get_error_stack(e);
    private get_frame_error_metadata;
    report_error_and_stop(e: Catastrophe | Error, caught_at: string): void;
    report_warning(e: Catastrophe): void;
    set_entity_known(entity_id: ID): void;
    get_entity_state(entity_id: ID): EntityState;
    get_random_number(): number;
    private generate_random_id(prefix?, entropy_size?);
    get_compiled_block_by_interpreter_id(rbid: ID): CompiledBlock;
    set_running_block(rbid: ID, block_id: ID): void;
}
