"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var rlt = require("resul-ts");
var _clone = require("lodash/clone");
var _cloneDeep = require("lodash/cloneDeep");
var _defaults = require("lodash/defaults");
var _filter = require("lodash/filter");
var _identity = require("lodash/identity");
var _includes = require("lodash/includes");
var _remove = require("lodash/remove");
var _some = require("lodash/some");
var _sortBy = require("lodash/sortBy");
var _sortedUniq = require("lodash/sortedUniq");
var _toString = require("lodash/toString");
var _uniq = require("lodash/uniq");
var catastrophic_1 = require("catastrophic");
var inversify_1 = require("inversify");
var di_symbols_1 = require("./di_symbols");
var basic_types_1 = require("./basic_types");
var block_types_1 = require("./block_types");
var H = require("./di_interfaces");
var block_provider_1 = require("./block_provider");
var RuntimeManagerImpl = (function () {
    function RuntimeManagerImpl(state_store, bif, orf, block_pool, block_registry, event_bus, log, ohno, prng_factory, runtime_data, task_manager, u, config) {
        var _this = this;
        this.state_store = state_store;
        this.bif = bif;
        this.orf = orf;
        this.block_pool = block_pool;
        this.block_registry = block_registry;
        this.event_bus = event_bus;
        this.log = log;
        this.ohno = ohno;
        this.prng_factory = prng_factory;
        this.runtime_data = runtime_data;
        this.task_manager = task_manager;
        this.u = u;
        this.config = config;
        this.frames = 0;
        this.interpreters_spawned = 0;
        this.timer_block_date = 0;
        this.entity_states = {};
        this.interpreters = {};
        this.sorted_interpreters = [];
        this.running_interpreter_was_blocked = false;
        this.interpreters_to_restart_when_finished = {};
        this.action_responders = {};
        this.state_responders = {};
        this.dynamic_responders = {};
        this.action_queue = [];
        this.action_parameters = {};
        this.typeclasses = {};
        this.procedure_compiled_block_map = {};
        this.variable_specs = {};
        this.script_variable_specs = {};
        this.variables = {};
        this.entity_variables = {};
        this.entity_id_to_interpreter_id_dict = {};
        this.interpreter_id_to_entity_id = {};
        this.entity_id_to_compiled_blocks = {};
        this.entity_id_to_typeclass_id = {};
        this.running_constructor_to_typeclass = {};
        this.running_destructor_to_typeclass = {};
        this.tell_source_map = {};
        this.responder_id_to_instance_interp_id = {};
        this.instance_interp_id_to_responder_id = {};
        this.just_restart = false;
        this.just_stopped = false;
        this.interpreters_needing_dispose = {};
        this.entities_needing_add = [];
        this.interpreters_needing_spawn = [];
        this.entities_needing_destruct = [];
        this.entities_needing_dispose = [];
        this.entities_cloned_times = {};
        this.running_group = undefined;
        this.running_group_changed = false;
        this.get_frame_error_metadata = function (error_metadata) {
            error_metadata = _cloneDeep(error_metadata);
            var error_rbid = error_metadata.source_map_rbid;
            var source_interpreter_id = error_metadata.interpreter_id;
            var source_rbid = error_metadata.source_map_rbid;
            var source_entity_id = error_metadata.source_entity_id;
            var source_block_id = error_metadata.block_id;
            if (source_interpreter_id !== undefined) {
                var tell_source = _this.tell_source_map[source_interpreter_id];
                if (tell_source !== undefined) {
                    source_block_id = tell_source.block_id;
                    source_entity_id = tell_source.entity_id;
                    source_rbid = tell_source.root_block_id;
                }
            }
            if (error_metadata.block_id != undefined
                && error_metadata.block_id == source_interpreter_id
                && source_block_id != undefined) {
                error_metadata.block_id = source_block_id;
            }
            var responder_rbid = _this.instance_interp_id_to_responder_id[source_rbid];
            if (responder_rbid != undefined) {
                source_rbid = responder_rbid;
            }
            if (error_metadata.proc_id !== undefined) {
                var procedure_container = _this.get_procedure(error_metadata.proc_id);
                if (procedure_container !== undefined) {
                    source_rbid = procedure_container.script.id;
                    source_entity_id = procedure_container.source_entity_id;
                }
            }
            error_metadata.source_map_rbid = source_rbid;
            if (source_entity_id != undefined) {
                error_metadata.source_entity_id = source_entity_id;
            }
            return error_metadata;
        };
        this.event_bus.clones.dispose_clone.immediate.sub(function (remove_id) {
            _this.destruct_entity(remove_id);
        });
        var configure_runtime_manager = function () {
            var cfg = config.get();
            _this.deterministic = cfg.deterministic;
            _this.entity_max_clones_per_frame = cfg.entity_max_clones_per_frame;
            if (_this.deterministic == undefined) {
                _this.prng = prng_factory.create();
                _this.interpreter_sorters = _this.get_interpreter_sorters(false);
            }
            else {
                _this.prng = prng_factory.create(_this.deterministic.prng_seed);
                _this.interpreter_sorters = _this.get_interpreter_sorters(true);
            }
            if (_this.user_debug_mode != cfg.user_debug_mode && _this.runtime_data.is_running()) {
                _this.event_bus.warning.all.send({
                    error: _this.ohno.configuration.tried_to_change_user_debug_mode_while_running(),
                });
                _this.stop();
            }
            else {
                _this.user_debug_mode = cfg.user_debug_mode;
                _this.interpreter_factory = cfg.user_debug_mode ? _this.bif : _this.orf;
            }
        };
        configure_runtime_manager();
        this.event_bus.system.config_updated.immediate.sub(configure_runtime_manager);
    }
    RuntimeManagerImpl.prototype.set_variable_specs = function (variable_specs) {
        for (var i = 0; i < variable_specs.length; i++) {
            var spec = variable_specs[i];
            this.variable_specs[spec.id] = spec;
            if (spec.scope === basic_types_1.VariableScope.script) {
                this.script_variable_specs[spec.id] = spec;
                continue;
            }
            if (spec.scope === basic_types_1.VariableScope.entity) {
                if (!this.entity_variables[spec.entity_id]) {
                    this.entity_variables[spec.entity_id] = {};
                }
                this.entity_variables[spec.entity_id][spec.id] = spec.value;
                continue;
            }
            this.variables[spec.id] = spec.value;
        }
    };
    RuntimeManagerImpl.prototype.change_running_group = function (id) {
        if (id === this.running_group) {
            return;
        }
        this.running_group = id;
        this.running_group_changed = true;
    };
    RuntimeManagerImpl.prototype.interpreter_out_of_running_group = function (group_id) {
        if (group_id === undefined || this.running_group === undefined) {
            return false;
        }
        if (group_id === this.running_group) {
            return false;
        }
        return true;
    };
    RuntimeManagerImpl.prototype.block_finish_out_of_run_group = function (ns_id) {
        return this.block_registry.block_finish_out_of_run_group(ns_id);
    };
    RuntimeManagerImpl.prototype.clear = function () {
        this.frames = 0;
        this.interpreters_spawned = 0;
        this.timer_block_date = 0;
        this.entity_states = {};
        this.interpreters = {};
        this.sorted_interpreters = [];
        this.interpreters_needing_spawn = [];
        this.interpreters_to_restart_when_finished = {};
        this.action_responders = {};
        this.state_responders = {};
        this.dynamic_responders = {};
        this.action_queue = [];
        this.typeclasses = {};
        this.procedure_compiled_block_map = {};
        this.variable_specs = {};
        this.script_variable_specs = {};
        this.variables = {};
        this.entity_variables = {};
        this.entity_id_to_interpreter_id_dict = {};
        this.interpreter_id_to_entity_id = {};
        this.tell_source_map = {};
        for (var entity_id in this.entity_id_to_compiled_blocks) {
            var compiled_blocks = this.entity_id_to_compiled_blocks[entity_id];
            for (var rbid in compiled_blocks) {
                this.block_pool.release(compiled_blocks[rbid]);
            }
        }
        this.entity_id_to_compiled_blocks = {};
        this.entity_id_to_typeclass_id = {};
        this.running_constructor_to_typeclass = {};
        this.running_destructor_to_typeclass = {};
        this.responder_id_to_instance_interp_id = {};
        this.instance_interp_id_to_responder_id = {};
        this.just_restart = false;
        this.just_stopped = false;
        this.interpreters_needing_dispose = {};
        this.entities_needing_add = [];
        this.entities_needing_destruct = [];
        this.entities_needing_dispose = [];
        this.bif.clear();
        this.orf.clear();
        this.runtime_data.clear();
        this.state_store.clear();
        this.task_manager.clear();
        this.running_group = undefined;
    };
    RuntimeManagerImpl.prototype.do_spawn_interpreter = function (r) {
        var target_entity_id = r.identities.target_entity;
        var interp_id = r.identities.interpreter_id;
        if (this.entity_id_to_interpreter_id_dict[target_entity_id] != undefined) {
            this.entity_id_to_interpreter_id_dict[target_entity_id][interp_id] = true;
        }
        else {
            this.entity_id_to_interpreter_id_dict[target_entity_id] = (_a = {}, _a[interp_id] = true, _a);
        }
        this.interpreter_id_to_entity_id[interp_id] = target_entity_id;
        var interpreter = this.interpreter_factory.create(this, r.identities, {
            creation_counter: this.interpreters_spawned,
            frame_created: this.frames,
            responder_priority: r.responder_priority != undefined ? r.responder_priority : 50,
        }, r.compile_cache_id || r.identities.source_map_rbid, r.script, _cloneDeep(this.script_variable_specs), r.group_id, r.is_warped || false, r.action_parameters, r.on_finished);
        this.interpreters_spawned += 1;
        this.interpreters[interp_id] = interpreter;
        this.sorted_interpreters.push(interpreter);
        this.interpreters_to_restart_when_finished[interp_id] = this.block_registry.block_restart_when_finished(r.script.type);
        var _a;
    };
    RuntimeManagerImpl.prototype.spawn_interpreter = function (r) {
        if (this.runtime_data.is_running()) {
            this.interpreters_needing_spawn.push(r);
        }
        else {
            this.do_spawn_interpreter(r);
        }
    };
    RuntimeManagerImpl.prototype.get_responder_filter_value = function (param) {
        if (param == undefined) {
            return param;
        }
        if (!this.u.block.is.compiled_block(param)) {
            return _toString(param);
        }
        else if (this.u.block.is.atomic_type(param.type)) {
            return _toString(param.params[Object.keys(param.params)[0]]);
        }
        else {
            throw this.ohno.system.unknown_action_block_param_type({
                caught_at: 'RuntimeManager::get_param_value, unknown action block param type',
            });
        }
    };
    RuntimeManagerImpl.prototype.load = function (ces) {
        for (var i = 0; i < ces.length; i++) {
            var ce = ces[i];
            for (var proc_name in ce.procedures) {
                var proc = ce.procedures[proc_name];
                this.procedure_load(ce.id, proc_name, proc);
            }
        }
        var results = [];
        for (var i = 0; i < ces.length; i++) {
            results.push(this.load_typeclass(ces[i]));
        }
        return results;
    };
    RuntimeManagerImpl.prototype.load_typeclass = function (ce) {
        var typeclass_id = ce.id;
        var runnables = [];
        var constructor;
        var destructor;
        var n_constructors = 0;
        var n_destructors = 0;
        for (var source_rbid in ce.compiled_block_map) {
            var script = ce.compiled_block_map[source_rbid];
            var type = script.type;
            var resp_info = this.block_registry.get_responder_info(type);
            var constructor_or_destructor = false;
            var group_id = (ce.running_group_id) ? ce.running_group_id[source_rbid] : undefined;
            var runnable = {
                identities: {
                    typeclass_id: typeclass_id,
                    source_map_entity: ce.id,
                    source_map_rbid: source_rbid,
                    target_entity: ce.id,
                    interpreter_id: source_rbid,
                },
                script: script,
                group_id: group_id,
            };
            if (resp_info != undefined) {
                var resp_type = resp_info.responder_spec.type;
                if (resp_type == block_provider_1.ResponderType.Constructor) {
                    constructor = runnable;
                    n_constructors += 1;
                    constructor_or_destructor = true;
                }
                if (resp_type == block_provider_1.ResponderType.Destructor) {
                    destructor = runnable;
                    n_destructors += 1;
                    constructor_or_destructor = true;
                }
            }
            if (constructor_or_destructor == false) {
                runnables.push(runnable);
            }
        }
        if (n_constructors > 1) {
            this.report_error_and_stop(this.ohno.compiler.user.defined_multiple_constructors({
                typeclass_id: typeclass_id,
                compiled_entity: ce,
            }), "RuntimeManager::load_typeclass");
            return rlt.fail('Typeclass had multiple constructors');
        }
        if (n_destructors > 1) {
            this.report_error_and_stop(this.ohno.compiler.user.defined_multiple_destructors({
                compiled_entity: ce,
            }), "RuntimeManager::load_typeclass");
            return rlt.fail('Typeclass had multiple destructors');
        }
        this.typeclasses[typeclass_id] = {
            constructor: constructor,
            destructor: destructor,
            runnables: runnables,
        };
        return rlt.ok();
    };
    RuntimeManagerImpl.prototype.create_singleton_entity_instances = function () {
        var results = [];
        for (var typeclass_id in this.typeclasses) {
            var res = this.create_entity_instance(typeclass_id, typeclass_id, undefined);
            results = results.concat(res);
        }
        return results;
    };
    RuntimeManagerImpl.prototype.create_entity_instance = function (typeclass_id, entity_id, params) {
        if (this.entity_id_to_typeclass_id[entity_id] != undefined) {
            return [rlt.fail('An entity with that id already exists')];
        }
        var typeclass = this.typeclasses[typeclass_id];
        if (typeclass == undefined) {
            return [rlt.fail("No typeclass with id " + typeclass_id)];
        }
        var _a = this.typeclasses[typeclass_id], constructor = _a.constructor, runnables = _a.runnables;
        this.entity_id_to_typeclass_id[entity_id] = typeclass_id;
        this.set_entity_known(entity_id);
        if (constructor != undefined) {
            var specialized = this.specialize_runnable(constructor, entity_id, params);
            var init_res = this.initialize_runnable(specialized, undefined, true, false);
            if (rlt.is_ok(init_res)) {
                this.running_constructor_to_typeclass[init_res.result] = typeclass_id;
            }
            return [
                init_res,
            ];
        }
        return this.initialize_entity_runnables(typeclass_id, entity_id);
    };
    RuntimeManagerImpl.prototype.initialize_entity_runnables = function (typeclass_id, entity_id) {
        var runnables = this.typeclasses[typeclass_id].runnables;
        var results = [];
        for (var i = 0; i < runnables.length; i++) {
            var r = runnables[i];
            results.push(this.initialize_runnable(r, entity_id));
        }
        return results;
    };
    RuntimeManagerImpl.prototype.specialize_runnable = function (r, target_entity, action_parameters) {
        var typeclass_id = r.identities.typeclass_id;
        var source_map_rbid = r.identities.source_map_rbid;
        var verbose_id = "__typeclass-" + typeclass_id + "__instance-" + target_entity + "__srbid-" + source_map_rbid + "__";
        return {
            identities: {
                typeclass_id: typeclass_id,
                source_map_entity: r.identities.source_map_entity,
                source_map_rbid: source_map_rbid,
                target_entity: target_entity,
                interpreter_id: this.generate_random_id(verbose_id),
            },
            script: r.script,
            action_parameters: action_parameters,
            group_id: r.group_id,
        };
    };
    RuntimeManagerImpl.prototype.initialize_runnable = function (r, target_entity, is_constructor, is_destructor) {
        if (target_entity != undefined) {
            r = this.specialize_runnable(r, target_entity);
        }
        var res_info = this.block_registry.get_responder_info(r.script.type);
        var source_map_entity = r.identities.source_map_entity;
        var original_id = this.runtime_data.clone_id_2_original_id(r.identities.target_entity);
        if (original_id == undefined) {
            if (is_destructor == false) {
                this.set_entity_known(source_map_entity);
            }
            if (!this.entity_id_to_compiled_blocks[source_map_entity]) {
                this.entity_id_to_compiled_blocks[source_map_entity] = {};
            }
            this.entity_id_to_compiled_blocks[source_map_entity][r.identities.source_map_rbid] = this.block_pool.clone(r.script);
        }
        if (res_info == undefined || is_constructor || is_destructor) {
            if (this.user_debug_mode) {
                r.script = this.block_pool.clone(r.script);
            }
            else {
                r.script = this.entity_id_to_compiled_blocks[r.identities.source_map_entity][r.identities.source_map_rbid];
            }
            try {
                this.spawn_interpreter(r);
            }
            catch (e) {
                this.report_error_and_stop(e, 'RuntimeManager::initialize_runnable, spawning interpreter');
                return rlt.fail('Error while spawning interpreter');
            }
            return rlt.success(r.identities.interpreter_id);
        }
        var action_spec = res_info.action_spec, responder_spec = res_info.responder_spec, namespace = res_info.namespace;
        var ar = r;
        ar.action_spec = action_spec;
        ar.responder_spec = responder_spec;
        ar.namespace = namespace;
        ar.event_id = action_spec.id;
        var value_filter_arg_name = 'filter_value';
        var sub_type_filter_arg_name = 'filter_sub_type';
        if (responder_spec.filter_arg_names != undefined) {
            value_filter_arg_name = responder_spec.filter_arg_names.value;
            sub_type_filter_arg_name = responder_spec.filter_arg_names.sub_type;
        }
        var value = this.get_responder_filter_value(r.script.params[value_filter_arg_name]);
        if (value) {
            ar.value_filter = value;
        }
        var sub_type = this.get_responder_filter_value(r.script.params[sub_type_filter_arg_name]);
        if (sub_type) {
            ar.sub_type_filter = sub_type;
        }
        var collection = this.action_responders;
        if (ar.responder_spec.type == block_provider_1.ResponderType.State) {
            collection = this.state_responders;
        }
        if (ar.responder_spec.type == block_provider_1.ResponderType.Dynamic) {
            collection = this.dynamic_responders;
        }
        var action_id = block_provider_1.namespaced_id(ar.namespace, ar.event_id);
        if (collection[action_id] == undefined) {
            collection[action_id] = [];
        }
        collection[action_id].push(ar);
        return rlt.success('responder');
    };
    RuntimeManagerImpl.prototype.send_action = function (action) {
        if (this.runtime_data.is_stopped()) {
            return;
        }
        this.action_queue.push(action);
    };
    RuntimeManagerImpl.prototype.runnable_should_respond_to_action = function (r, action) {
        if (r.action_spec.entity_specific) {
            if (action.entity_id == undefined
                || r.identities.target_entity != action.entity_id) {
                return false;
            }
        }
        if (r.sub_type_filter != undefined
            && r.sub_type_filter != action.sub_type) {
            return false;
        }
        if (r.value_filter != undefined) {
            var event_value = action.value;
            if (event_value == undefined) {
                if (r.action_spec.statefulness != undefined) {
                    event_value = r.action_spec.statefulness.default_value;
                }
            }
            if (r.value_filter != event_value) {
                return false;
            }
        }
        return true;
    };
    RuntimeManagerImpl.prototype.create_action_responder_interpreters = function (actions) {
        for (var i = 0; i < actions.length; i++) {
            var action = actions[i];
            var action_id = block_provider_1.namespaced_id(action.namespace, action.id);
            if (this.action_responders[action_id] == undefined) {
                continue;
            }
            for (var j = 0; j < this.action_responders[action_id].length; j++) {
                var r = this.action_responders[action_id][j];
                if (r.responder_spec.async == false) {
                    if (this.responder_id_to_instance_interp_id[r.identities.interpreter_id] != undefined) {
                        continue;
                    }
                }
                if (!this.runnable_should_respond_to_action(r, action)) {
                    continue;
                }
                this.do_add_responder_interpreters(r, 'action', action.parameters);
            }
        }
    };
    RuntimeManagerImpl.prototype.create_state_responder_interpreters = function () {
        for (var event_id in this.state_responders) {
            var responders = this.state_responders[event_id];
            for (var i = 0; i < responders.length; i++) {
                var r = responders[i];
                if (r.responder_spec.async == false) {
                    if (this.responder_id_to_instance_interp_id[r.identities.interpreter_id] != undefined) {
                        continue;
                    }
                }
                var query = {
                    action_namespace: r.namespace,
                    action_id: r.action_spec.id,
                    sub_type: r.sub_type_filter,
                };
                if (r.action_spec.entity_specific) {
                    query.entity_id = r.identities.interpreter_id;
                }
                var state_value = this.state_store.get_action_state_value(query);
                if (r.value_filter != undefined && state_value != r.value_filter) {
                    continue;
                }
                this.do_add_responder_interpreters(r, 'state');
            }
        }
    };
    RuntimeManagerImpl.prototype.is_stateful_action = function (action_id) {
        var action_spec = this.block_registry.get_action_spec(action_id);
        if (action_spec == undefined) {
            return false;
        }
        return action_spec.statefulness != undefined;
    };
    RuntimeManagerImpl.prototype.create_dynamic_responder_interpreters = function (actions) {
        var _this = this;
        var filtered_action_id = _filter(Object.keys(this.dynamic_responders), function (action_id) {
            return _some(actions, { id: action_id }) || _this.is_stateful_action(action_id);
        });
        for (var i = 0; i < filtered_action_id.length; i++) {
            var responders = this.dynamic_responders[filtered_action_id[i]];
            for (var j = 0; j < responders.length; j++) {
                var r = responders[j];
                if (r.responder_spec.trigger_function == undefined) {
                    continue;
                }
                if (r.responder_spec.async == false) {
                    if (this.responder_id_to_instance_interp_id[r.identities.interpreter_id] != undefined) {
                        continue;
                    }
                }
                var should_trigger = r.responder_spec.trigger_function(actions, r.value_filter, r.sub_type_filter, r.identities.target_entity);
                if (!should_trigger) {
                    continue;
                }
                this.do_add_responder_interpreters(r, 'dynamic');
            }
        }
    };
    RuntimeManagerImpl.prototype.do_add_responder_interpreters = function (r, type, action_parameters) {
        if (this.interpreter_out_of_running_group(r.group_id)) {
            return;
        }
        var new_interpreter_id = this.generate_random_id("__" + type + "_responder_interpreter__");
        var new_identities = _clone(r.identities);
        new_identities.interpreter_id = new_interpreter_id;
        var responder_id = r.identities.interpreter_id;
        this.instance_interp_id_to_responder_id[new_interpreter_id] = responder_id;
        if (this.responder_id_to_instance_interp_id[responder_id] == undefined) {
            this.responder_id_to_instance_interp_id[responder_id] = [new_interpreter_id];
        }
        else {
            this.responder_id_to_instance_interp_id[responder_id].push(new_interpreter_id);
        }
        var script;
        if (this.user_debug_mode) {
            script = this.block_pool.clone(r.script);
            script.id = new_interpreter_id;
        }
        else {
            script = r.script;
        }
        this.spawn_interpreter({
            identities: new_identities,
            script: script,
            action_parameters: action_parameters,
            responder_priority: r.responder_priority,
            group_id: r.group_id,
        });
    };
    RuntimeManagerImpl.prototype.update_actions = function () {
        this.create_action_responder_interpreters(this.action_queue);
        this.state_store.update(this.action_queue);
        this.create_state_responder_interpreters();
        this.create_dynamic_responder_interpreters(this.action_queue);
        this.action_queue = [];
    };
    RuntimeManagerImpl.prototype.do_add_entity = function (e) {
        var compiled_block_rbids = Object.keys(e.compiled_block_map);
        if (this.deterministic != undefined) {
            compiled_block_rbids = _sortBy(compiled_block_rbids, _identity);
        }
        this.entity_id_to_typeclass_id[e.entity_id] = e.typeclass_id;
        for (var i = 0; i < compiled_block_rbids.length; i++) {
            var rbid = compiled_block_rbids[i];
            this.initialize_runnable({
                identities: {
                    typeclass_id: e.typeclass_id,
                    source_map_entity: e.source_map_entity,
                    source_map_rbid: e.source_rbids[rbid],
                    target_entity: e.entity_id,
                    interpreter_id: rbid,
                },
                script: e.compiled_block_map[rbid],
                group_id: e.running_group_id[rbid],
            });
        }
    };
    RuntimeManagerImpl.prototype.add_entity = function (e) {
        if (this.runtime_data.is_running()) {
            this.entities_needing_add.push(e);
        }
        else {
            this.do_add_entity(e);
        }
    };
    RuntimeManagerImpl.prototype.get_cloned_entity_data = function (entity_id, is_mirror) {
        var compiled_block_map = this.entity_id_to_compiled_blocks[entity_id];
        if (compiled_block_map === undefined) {
            compiled_block_map = {};
        }
        var new_entity_id = this.generate_random_id("_clone_" + entity_id + "_random_id");
        var entity_to_add = {
            typeclass_id: entity_id,
            entity_id: new_entity_id,
            source_map_entity: entity_id,
            compiled_block_map: {},
            source_rbids: {},
            running_group_id: {},
        };
        var rbids = Object.keys(compiled_block_map);
        for (var i = 0; i < rbids.length; i++) {
            var root_block_id = rbids[i];
            var compiled_block = compiled_block_map[root_block_id];
            if (is_mirror && block_types_1.EVENT_BLOCKS[compiled_block.type] != block_types_1.EVENT_BLOCKS.start_as_a_mirror) {
                continue;
            }
            var new_block_id = root_block_id + this.generate_random_id('_random_block_id');
            var script = void 0;
            if (this.user_debug_mode) {
                script = this.block_pool.clone(compiled_block);
                script.id = new_block_id;
            }
            else {
                script = compiled_block;
            }
            entity_to_add.source_rbids[new_block_id] = root_block_id;
            entity_to_add.compiled_block_map[new_block_id] = script;
        }
        return entity_to_add;
    };
    RuntimeManagerImpl.prototype.clone_entity = function (entity_id, is_mirror) {
        if (this.entity_states[entity_id] === basic_types_1.EntityState.Unknown) {
            throw this.ohno.user.clone_unknown_entity({
                entity_id: entity_id,
            });
        }
        if (this.entities_cloned_times[entity_id] == undefined) {
            this.entities_cloned_times[entity_id] = 0;
        }
        if (this.entities_cloned_times[entity_id] > this.entity_max_clones_per_frame) {
            return;
        }
        this.entities_cloned_times[entity_id]++;
        var typeclass_id = entity_id;
        var entity_to_add = this.get_cloned_entity_data(entity_id, is_mirror);
        var cloned_variables = _cloneDeep(this.entity_variables[entity_id]);
        this.entity_variables[entity_to_add.entity_id] = cloned_variables;
        this.runtime_data.clone_created(entity_id, entity_to_add.entity_id);
        if (is_mirror) {
            var rbids = Object.keys(entity_to_add.compiled_block_map);
            for (var i = 0; i < rbids.length; i++) {
                this.runtime_data.set_mirror(rbids[i]);
            }
        }
        this.add_entity(entity_to_add);
        return entity_to_add.entity_id;
    };
    RuntimeManagerImpl.prototype.run = function () {
        this.timer_block_date = new Date().valueOf();
        this.runtime_data.set_running();
        this.event_bus.runtime_manager.start.send();
    };
    RuntimeManagerImpl.prototype.delete_other_interpreters = function (interpreter_id) {
        var other_interpreters = Object.keys(this.interpreters)
            .filter(function (i) { return i != interpreter_id; });
        for (var i = 0; i < other_interpreters.length; i++) {
            this.dispose_block_group(other_interpreters[i]);
        }
    };
    RuntimeManagerImpl.prototype.stop = function () {
        if (this.runtime_data.is_stopped()) {
            return;
        }
        this.runtime_data.set_stopped();
        this.just_stopped = true;
    };
    RuntimeManagerImpl.prototype.restart = function () {
        this.just_restart = true;
    };
    RuntimeManagerImpl.prototype.sort_interpreters = function () {
        this.sorted_interpreters = _sortBy(this.sorted_interpreters, this.interpreter_sorters);
    };
    RuntimeManagerImpl.prototype.update = function () {
        var _this = this;
        this.event_bus.runtime_manager.before_update.send();
        if (this.entities_needing_add.length !== 0
            && this.entities_needing_destruct.length !== 0) {
            _remove(this.entities_needing_add, function (e) { return _includes(_this.entities_needing_destruct, e.entity_id); });
        }
        try {
            this.update_dispose();
        }
        catch (e) {
            this.report_error_and_stop(e, 'RuntimeManager::update, update_dispose');
            return;
        }
        if (this.just_stopped || this.just_restart) {
            var to_stop = this.just_stopped;
            this.clear();
            this.event_bus.runtime_data._meta.clear();
            this.event_bus.runtime_manager._meta.clear();
            if (to_stop) {
                this.event_bus.runtime_manager.stop.send();
            }
            else {
                this.event_bus.runtime_manager.restart.send();
            }
            return;
        }
        if (this.runtime_data.is_running()) {
            try {
                var n_interpreters = this.sorted_interpreters.length;
                for (var i = 0; i < this.entities_needing_add.length; i++) {
                    this.do_add_entity(this.entities_needing_add[i]);
                }
                this.entities_needing_add = [];
                this.update_actions();
                for (var i = 0; i < this.interpreters_needing_spawn.length; i++) {
                    this.do_spawn_interpreter(this.interpreters_needing_spawn[i]);
                }
                this.interpreters_needing_spawn = [];
                if (this.sorted_interpreters.length != n_interpreters) {
                    this.sort_interpreters();
                }
            }
            catch (e) {
                this.report_error_and_stop(e, 'RuntimeManager::update, adding and sorting interpreters');
                return;
            }
            try {
                this.step_code();
            }
            catch (e) {
                this.report_error_and_stop(e, 'RuntimeManager::update, stepping code');
                return;
            }
            this.task_manager.update();
            this.frames++;
            this.event_bus.runtime_manager.after_update.send();
        }
    };
    RuntimeManagerImpl.prototype.set_current_interpreter_not_blocked = function () {
        this.running_interpreter_was_blocked = false;
    };
    RuntimeManagerImpl.prototype.current_interpreter_must_yield = function (interpreter_id, group_id) {
        var needs_dispose = this.interpreters_needing_dispose[interpreter_id] != undefined &&
            this.interpreters_needing_dispose[interpreter_id];
        var out_of_run_group = this.running_group_changed && this.interpreter_out_of_running_group(group_id);
        return this.running_interpreter_was_blocked || needs_dispose || out_of_run_group;
    };
    RuntimeManagerImpl.prototype.add_task = function (t) {
        if (t.blocking) {
            this.running_interpreter_was_blocked = true;
        }
        return this.task_manager.add_task(this.generate_random_id('runtime_task'), t);
    };
    RuntimeManagerImpl.prototype.get_thread_lock = function (entity_id, interpreter_id) {
        return this.add_task({
            entity_id: entity_id,
            interpreter_id: interpreter_id,
            blocking: true,
        });
    };
    RuntimeManagerImpl.prototype.thread_wait = function (entity_id, interpreter_id, lifetime) {
        this.add_task({
            entity_id: entity_id,
            interpreter_id: interpreter_id,
            lifetime: lifetime,
            blocking: true,
        });
    };
    RuntimeManagerImpl.prototype.procedure_load = function (source_entity_id, procedure_name, compiled_block) {
        this.procedure_compiled_block_map[procedure_name] = {
            name: procedure_name,
            script: compiled_block,
            source_entity_id: source_entity_id,
        };
    };
    RuntimeManagerImpl.prototype.get_procedure = function (p_name, do_clone) {
        if (do_clone === void 0) { do_clone = true; }
        var procedure_container = this.procedure_compiled_block_map[p_name];
        if (procedure_container == undefined) {
            return;
        }
        if (do_clone) {
            procedure_container.script = this.block_pool.clone(procedure_container.script);
            procedure_container.script.id = this.generate_random_id('procedure_id');
        }
        return procedure_container;
    };
    RuntimeManagerImpl.prototype.get_elapsed_frames = function () {
        return this.frames;
    };
    RuntimeManagerImpl.prototype.reset_timer = function () {
        this.timer_block_date = new Date().valueOf();
    };
    RuntimeManagerImpl.prototype.get_timer_elapsed_s = function () {
        return (new Date().valueOf() - this.timer_block_date) / 1000;
    };
    RuntimeManagerImpl.prototype.do_dispose_of_interpreter = function (interp_id, force_dispose) {
        if (force_dispose === void 0) { force_dispose = false; }
        if (this.interpreters[interp_id] == undefined) {
            this.u.log.warn("Attempted to dispose non-existing interpreter.");
            return;
        }
        var interpreter = this.interpreters[interp_id];
        if (!force_dispose && this.interpreters_to_restart_when_finished[interp_id]) {
            interpreter.reset();
            this.runtime_data.dispose_interpreter_data(interp_id);
            this.task_manager.dispose_tasks_given({ interpreter_id: interp_id });
            return;
        }
        var entity_id = this.interpreter_id_to_entity_id[interp_id];
        var responder_rbid = this.instance_interp_id_to_responder_id[interp_id];
        if (responder_rbid != undefined) {
            var responder_instances = this.responder_id_to_instance_interp_id[responder_rbid];
            if (responder_instances.length == 1) {
                delete (this.responder_id_to_instance_interp_id[responder_rbid]);
            }
            else {
                _remove(responder_instances, function (r) { return r == interp_id; });
            }
            delete (this.instance_interp_id_to_responder_id[interp_id]);
        }
        interpreter.dispose();
        delete (this.interpreters[interp_id]);
        delete (this.entity_id_to_interpreter_id_dict[entity_id][interp_id]);
        delete (this.interpreters_to_restart_when_finished[interp_id]);
        _remove(this.sorted_interpreters, function (i) { return i === interpreter; });
        delete (this.interpreter_id_to_entity_id[interp_id]);
        delete (this.tell_source_map[interp_id]);
        this.runtime_data.dispose_interpreter_data(interp_id);
        this.task_manager.dispose_tasks_given({ interpreter_id: interp_id });
        var ctor_typeclass_id = this.running_constructor_to_typeclass[interp_id];
        if (ctor_typeclass_id != undefined) {
            delete (this.running_constructor_to_typeclass[interp_id]);
            this.initialize_entity_runnables(ctor_typeclass_id, entity_id);
            return;
        }
        var dtor_typeclass_id = this.running_destructor_to_typeclass[interp_id];
        if (dtor_typeclass_id != undefined) {
            delete (this.running_destructor_to_typeclass[interp_id]);
            this.entities_needing_dispose.push(entity_id);
        }
    };
    RuntimeManagerImpl.prototype.do_dispose_of_entity = function (entity_id) {
        if (this.entity_states[entity_id] !== undefined) {
            this.entity_states[entity_id] = basic_types_1.EntityState.Disposed;
        }
        this.runtime_data.entity_disposed(entity_id);
        delete (this.entity_id_to_interpreter_id_dict[entity_id]);
        delete (this.entity_variables[entity_id]);
        delete (this.entity_id_to_typeclass_id[entity_id]);
        this.task_manager.dispose_tasks_given({ entity_id: entity_id });
    };
    RuntimeManagerImpl.prototype.do_destruct_entity = function (entity_id) {
        if (this.entity_states[entity_id] !== undefined) {
            this.entity_states[entity_id] = basic_types_1.EntityState.Destructing;
        }
        var rbids_dict = this.entity_id_to_interpreter_id_dict[entity_id];
        if (rbids_dict !== undefined) {
            var rbids_to_dispose = Object.keys(rbids_dict);
            for (var i = 0; i < rbids_to_dispose.length; i++) {
                var rbid = rbids_to_dispose[i];
                this.do_dispose_of_interpreter(rbid, true);
            }
        }
        var matches_entity_to_dispose = function (r) { return r.identities.target_entity == entity_id; };
        for (var r in this.action_responders) {
            _remove(this.action_responders[r], matches_entity_to_dispose);
        }
        for (var r in this.state_responders) {
            _remove(this.state_responders[r], matches_entity_to_dispose);
        }
        for (var r in this.dynamic_responders) {
            _remove(this.dynamic_responders[r], matches_entity_to_dispose);
        }
        var destructor;
        var typeclass_id = this.entity_id_to_typeclass_id[entity_id];
        var typeclass = this.typeclasses[typeclass_id];
        if (typeclass_id == undefined || typeclass == undefined) {
            this.report_warning(this.ohno.warning.entity_has_no_known_typeclass({
                typeclass_id: typeclass_id,
                entity_id: entity_id,
                caught_at: "RuntimeManager::do_destruct_entity",
            }));
        }
        else {
            if (typeclass != undefined) {
                destructor = typeclass.destructor;
            }
        }
        if (destructor == undefined) {
            this.do_dispose_of_entity(entity_id);
            return;
        }
        var init_res = this.initialize_runnable(destructor, entity_id, false, true);
        if (rlt.is_ok(init_res)) {
            this.running_destructor_to_typeclass[init_res.result] = typeclass_id;
        }
    };
    RuntimeManagerImpl.prototype.update_dispose = function () {
        for (var rbid in this.interpreters_needing_dispose) {
            this.do_dispose_of_interpreter(rbid);
        }
        this.interpreters_needing_dispose = {};
        if (this.deterministic != undefined) {
            this.entities_needing_destruct = _sortBy(this.entities_needing_destruct, _identity);
            this.entities_needing_destruct = _sortedUniq(this.entities_needing_destruct);
        }
        else {
            this.entities_needing_destruct = _uniq(this.entities_needing_destruct);
        }
        for (var i = 0; i < this.entities_needing_destruct.length; i++) {
            this.do_destruct_entity(this.entities_needing_destruct[i]);
        }
        this.entities_needing_destruct = [];
        if (this.deterministic != undefined) {
            this.entities_needing_dispose = _sortBy(this.entities_needing_dispose, _identity);
            this.entities_needing_dispose = _sortedUniq(this.entities_needing_dispose);
        }
        else {
            this.entities_needing_dispose = _uniq(this.entities_needing_dispose);
        }
        for (var i = 0; i < this.entities_needing_dispose.length; i++) {
            this.do_dispose_of_entity(this.entities_needing_dispose[i]);
        }
        this.entities_needing_dispose = [];
        this.task_manager.update_dispose();
    };
    RuntimeManagerImpl.prototype.get_interpreter_sorters = function (is_deterministic) {
        var responders_early = function (i) {
            return i.metadata.type == 'self_listen' ||
                block_types_1.EVENT_BLOCKS[i.metadata.type] == block_types_1.EVENT_BLOCKS.mouse_on_emit;
        };
        var box_break_block_last = function (i) {
            return block_types_1.EVENT_BLOCKS[i.metadata.type] != block_types_1.EVENT_BLOCKS.block_on_break;
        };
        var frame_order = function (i) { return i.metadata.priorities.frame_created; };
        var responder_order = function (i) { return i.metadata.priorities.responder_priority; };
        var creation_order = function (i) { return 0 - i.metadata.priorities.creation_counter; };
        if (is_deterministic) {
            return [
                box_break_block_last,
                frame_order,
                responder_order,
                creation_order,
            ];
        }
        return [
            box_break_block_last,
            frame_order,
            responder_order,
            creation_order,
        ];
    };
    RuntimeManagerImpl.prototype.step_code = function () {
        this.running_group_changed = false;
        this.entities_cloned_times = {};
        var to_step_stack = this.sorted_interpreters;
        var finished_stepping_stack = [];
        if (to_step_stack.length === 0) {
            this.event_bus.runtime_manager.idle.send();
            return;
        }
        while (true) {
            var interpreter = to_step_stack.pop();
            if (interpreter == undefined) {
                break;
            }
            finished_stepping_stack.push(interpreter);
            var interpreter_id = interpreter.metadata.interpreter_id;
            if (this.interpreters_needing_dispose[interpreter_id]) {
                continue;
            }
            if (this.interpreter_out_of_running_group(interpreter.metadata.group_id)) {
                if (this.block_finish_out_of_run_group(interpreter.metadata.type)) {
                    this.dispose_block_group(interpreter_id);
                }
                continue;
            }
            if (this.is_blocking(interpreter_id)) {
                continue;
            }
            this._step(interpreter_id, interpreter);
        }
        while (true) {
            var interp = finished_stepping_stack.pop();
            if (interp == undefined) {
                break;
            }
            this.sorted_interpreters.push(interp);
        }
    };
    RuntimeManagerImpl.prototype.is_blocking = function (interpreter_id) {
        return this.task_manager.is_blocking(interpreter_id);
    };
    RuntimeManagerImpl.prototype._step = function (interpreter_id, interpreter) {
        var step_result;
        try {
            step_result = interpreter.step();
        }
        catch (e) {
            var error_metadata = {
                root_block_id: interpreter_id,
            };
            if (e instanceof catastrophic_1.Catastrophe) {
                e.annotation = _defaults(e.annotation, error_metadata);
            }
            else {
                e = this.ohno.system.unknown_system_error(e, tslib_1.__assign({ caught_at: 'RuntimeManager::_step' }, error_metadata));
            }
            throw (e);
        }
        if (step_result == basic_types_1.StepResult.finished) {
            this.dispose_block_group(interpreter_id);
        }
        if (this.interpreter_out_of_running_group(interpreter.metadata.group_id)
            && this.block_finish_out_of_run_group(interpreter.metadata.type)) {
            this.dispose_block_group(interpreter_id);
        }
    };
    RuntimeManagerImpl.prototype.dispose_sprite = function (entity_id) {
        this.entities_needing_destruct.push(entity_id);
    };
    RuntimeManagerImpl.prototype.destruct_entity = function (entity_id) {
        this.entities_needing_destruct.push(entity_id);
    };
    RuntimeManagerImpl.prototype.dispose_block_group = function (interpreter_id) {
        var interpreter = this.interpreters[interpreter_id];
        if (!interpreter) {
            this.u.log.warn("Can't dispose block group (rbid: " + interpreter_id + ") without associated interpreter");
            return;
        }
        this.interpreters_needing_dispose[interpreter_id] = true;
    };
    RuntimeManagerImpl.prototype.dispose_all = function () {
        for (var entity_id in this.entity_id_to_interpreter_id_dict) {
            var interp_id_dict = this.entity_id_to_interpreter_id_dict[entity_id];
            for (var interp_id in interp_id_dict) {
                this.dispose_block_group(interp_id);
            }
        }
    };
    RuntimeManagerImpl.prototype.dispose_other_block_groups_of_entity = function (entity_id, interpreter_to_keep_id) {
        var interp_id_dict = this.entity_id_to_interpreter_id_dict[entity_id];
        if (interp_id_dict == undefined) {
            return;
        }
        for (var interp_id in interp_id_dict) {
            if (interp_id == interpreter_to_keep_id) {
                continue;
            }
            this.dispose_block_group(interp_id);
        }
    };
    RuntimeManagerImpl.prototype.dispose_block_groups_of_other_entities = function (entity_id) {
        for (var _entity_id in this.entity_id_to_interpreter_id_dict) {
            if (entity_id === _entity_id) {
                continue;
            }
            var interp_id_dict = this.entity_id_to_interpreter_id_dict[_entity_id];
            for (var interp_id in interp_id_dict) {
                this.dispose_block_group(interp_id);
            }
        }
    };
    RuntimeManagerImpl.prototype.get_var_or_list = function (var_id, default_value, interpreter_id, entity_id) {
        var var_spec = this.variable_specs[var_id];
        if (!var_spec) {
            if (!this.variables[var_id]) {
                this.variables[var_id] = default_value;
            }
            return this.variables[var_id];
        }
        if (var_spec.scope === basic_types_1.VariableScope.script) {
            if (interpreter_id == undefined) {
                throw this.ohno.system.called_get_variable_without_needed_parameters({
                    var_id: var_id,
                });
            }
            return this.interpreters[interpreter_id].get_variable(var_id);
        }
        if (var_spec.scope === basic_types_1.VariableScope.entity) {
            if (entity_id == undefined) {
                throw this.ohno.system.called_get_variable_without_needed_parameters({
                    var_id: var_id,
                });
            }
            if (this.entity_variables[entity_id] && (this.entity_variables[entity_id][var_id] !== undefined)) {
                return this.entity_variables[entity_id][var_id];
            }
            throw this.ohno.user.entity_variable_operation_out_of_scope({
                target_entity_id: entity_id,
                entity_variable_spec: var_spec,
            });
        }
        if (this.variables[var_id] == undefined) {
            this.variables[var_id] = default_value;
        }
        return this.variables[var_id];
    };
    RuntimeManagerImpl.prototype.get_list_id = function (list, entity_id) {
        var global_vars = this.variables;
        for (var id in global_vars) {
            var val = global_vars[id];
            if (val === list) {
                return id;
            }
        }
        var entity_vars = this.entity_variables[entity_id];
        for (var id in entity_vars) {
            var val = entity_vars[id];
            if (val === list) {
                return id;
            }
        }
        return;
    };
    RuntimeManagerImpl.prototype.set_variable = function (var_id, val, interpreter_id, entity_id) {
        var var_spec = this.variable_specs[var_id];
        if (var_spec == undefined || var_spec.scope == basic_types_1.VariableScope.global) {
            this.variables[var_id] = val;
            this.runtime_data.report_variable_updated(var_id, val);
            return;
        }
        if (var_spec.scope === basic_types_1.VariableScope.script) {
            if (interpreter_id == undefined) {
                throw this.ohno.system.called_set_variable_without_needed_parameters({
                    var_id: var_id,
                    val: val,
                });
            }
            this.interpreters[interpreter_id].set_variable(var_id, val);
            this.runtime_data.report_variable_updated(var_id, val);
            return;
        }
        if (var_spec.scope === basic_types_1.VariableScope.entity) {
            if (entity_id == undefined) {
                throw this.ohno.system.called_set_variable_without_needed_parameters({
                    var_id: var_id,
                    val: val,
                });
            }
            if (this.entity_variables[entity_id] && (this.entity_variables[entity_id][var_id] !== undefined)) {
                this.entity_variables[entity_id][var_id] = val;
                this.runtime_data.report_entity_variable_updated(var_id, val, entity_id);
                return;
            }
            throw this.ohno.user.entity_variable_operation_out_of_scope({
                target_entity_id: entity_id,
                entity_variable_spec: var_spec,
            });
        }
        this.variables[var_id] = val;
        this.runtime_data.report_variable_updated(var_id, val);
    };
    RuntimeManagerImpl.prototype.get_variable = function (var_id, interpreter_id, entity_id) {
        return this.get_var_or_list(var_id, 0, interpreter_id, entity_id);
    };
    RuntimeManagerImpl.prototype.lists_get = function (var_id, interpreter_id, entity_id) {
        return this.get_var_or_list(var_id, [], interpreter_id, entity_id);
    };
    RuntimeManagerImpl.prototype.get_global_variable = function (var_id) {
        return this.variables[var_id];
    };
    RuntimeManagerImpl.prototype.is_entity_variable = function (var_id) {
        var spec = this.variable_specs[var_id];
        if (!spec) {
            return false;
        }
        return spec.scope === basic_types_1.VariableScope.entity;
    };
    RuntimeManagerImpl.prototype.get_entity_id_from_root_block_id = function (interp_id) {
        return this.interpreter_id_to_entity_id[interp_id];
    };
    RuntimeManagerImpl.prototype.spawn_async_tell_interpreter = function (teller_identities, new_target_entity, script, group_id, is_warped) {
        this.do_spawn_tell_interpreter(teller_identities, new_target_entity, script, group_id, undefined, is_warped);
    };
    RuntimeManagerImpl.prototype.spawn_sync_tell_interpreter = function (teller_identities, new_target_entity, script, group_id) {
        var sync_lock = this.get_thread_lock(teller_identities.target_entity, teller_identities.interpreter_id);
        var on_finished = function () {
            sync_lock.stop();
        };
        this.do_spawn_tell_interpreter(teller_identities, new_target_entity, script, group_id, on_finished);
    };
    RuntimeManagerImpl.prototype.do_spawn_tell_interpreter = function (teller_identities, new_target_entity, script, group_id, on_finished, is_warped) {
        var parent_interpreter = this.interpreters[teller_identities.interpreter_id];
        if (is_warped == undefined && parent_interpreter.is_inside_warp != undefined) {
            is_warped = parent_interpreter.is_inside_warp();
        }
        var our_tell_source = {
            block_id: script.id,
            entity_id: teller_identities.source_map_entity,
            root_block_id: teller_identities.source_map_rbid,
            parent_stack: parent_interpreter.get_current_stack(),
        };
        var new_interpreter_id = this.generate_random_id('__tell_root_block__');
        this.tell_source_map[new_interpreter_id] = our_tell_source;
        if (this.user_debug_mode) {
            script.id = new_interpreter_id;
        }
        this.spawn_interpreter({
            compile_cache_id: script.id,
            identities: {
                typeclass_id: teller_identities.typeclass_id,
                source_map_entity: our_tell_source.entity_id,
                source_map_rbid: our_tell_source.root_block_id,
                target_entity: new_target_entity,
                interpreter_id: new_interpreter_id,
            },
            script: script,
            on_finished: on_finished,
            is_warped: is_warped,
            group_id: group_id,
        });
    };
    RuntimeManagerImpl.prototype.get_error_stack = function (e) {
        if (e.category.unique_code === 'HEART.COMPILER.SYSTEM'
            || e.category.unique_code === 'HEART.COMPILER.USER'
            || e.annotation.interpreter_id === undefined
            || e.annotation.interpreter_stack == undefined) {
            return [{
                    block_id: e.annotation.block_id,
                    source_entity_id: e.annotation.source_entity_id,
                    source_map_rbid: e.annotation.root_block_id,
                    interpreter_id: e.annotation.interpreter_id,
                    proc_id: e.annotation.proc_id,
                }];
        }
        var stack = _cloneDeep(e.annotation.interpreter_stack);
        var walk_interpreter = e.annotation.interpreter_id;
        while (true) {
            var tell_source = this.tell_source_map[walk_interpreter];
            if (tell_source === undefined) {
                break;
            }
            walk_interpreter = tell_source.root_block_id;
            var parent_stack = tell_source.parent_stack;
            var main_frame = stack.shift();
            parent_stack[parent_stack.length - 1].block_id = main_frame.block_id;
            parent_stack[parent_stack.length - 1].interpreter_id = main_frame.interpreter_id;
            stack = parent_stack.concat(stack);
        }
        stack[stack.length - 1].block_id = e.annotation.block_id;
        return stack;
    };
    RuntimeManagerImpl.prototype.report_error_and_stop = function (e, caught_at) {
        var wrapped_error;
        if (e instanceof catastrophic_1.Catastrophe) {
            wrapped_error = e;
        }
        else {
            wrapped_error = this.ohno.system.unknown_system_error(e, {
                caught_at: caught_at,
            });
        }
        var error_stack = this.get_error_stack(wrapped_error)
            .map(this.get_frame_error_metadata);
        wrapped_error.annotation.stack = error_stack;
        this.event_bus.error.runtime.send({
            error: wrapped_error,
            error_stack: error_stack,
        });
        this.stop();
    };
    RuntimeManagerImpl.prototype.report_warning = function (e) {
        var error_stack = this.get_error_stack(e)
            .map(this.get_frame_error_metadata);
        e.annotation.stack = error_stack;
        this.event_bus.warning.runtime.send({
            error: e,
            error_stack: error_stack,
        });
    };
    RuntimeManagerImpl.prototype.set_entity_known = function (entity_id) {
        this.entity_states[entity_id] = basic_types_1.EntityState.Known;
    };
    RuntimeManagerImpl.prototype.get_entity_state = function (entity_id) {
        var state = this.entity_states[entity_id];
        if (state == undefined) {
            return basic_types_1.EntityState.Unknown;
        }
        return state;
    };
    RuntimeManagerImpl.prototype.get_random_number = function () {
        return this.prng.random();
    };
    RuntimeManagerImpl.prototype.generate_random_id = function (prefix, entropy_size) {
        if (prefix === void 0) { prefix = ''; }
        if (entropy_size === void 0) { entropy_size = 9; }
        var id_array = [prefix];
        for (var i = 0; i < entropy_size; i++) {
            id_array.push(Math.round(10 * this.prng.random()));
        }
        return id_array.join('');
    };
    RuntimeManagerImpl.prototype.get_compiled_block_by_interpreter_id = function (rbid) {
        var entity_id = this.interpreter_id_to_entity_id[rbid];
        var original_compiled_block = this.entity_id_to_compiled_blocks[entity_id][rbid];
        return original_compiled_block;
    };
    RuntimeManagerImpl.prototype.set_running_block = function (rbid, block_id) {
        this.runtime_data.set_running_block(rbid, block_id);
    };
    RuntimeManagerImpl = tslib_1.__decorate([
        inversify_1.injectable(),
        tslib_1.__param(0, inversify_1.inject(di_symbols_1.BINDING.ActionStateStore)),
        tslib_1.__param(1, inversify_1.inject(di_symbols_1.BINDING.BlockInterpreterFactory)),
        tslib_1.__param(2, inversify_1.inject(di_symbols_1.BINDING.OptiRunnerFactory)),
        tslib_1.__param(3, inversify_1.inject(di_symbols_1.BINDING.BlockPool)),
        tslib_1.__param(4, inversify_1.inject(di_symbols_1.BINDING.BlockRegistry)),
        tslib_1.__param(5, inversify_1.inject(di_symbols_1.BINDING.EventBus)),
        tslib_1.__param(6, inversify_1.inject(di_symbols_1.BINDING.Log)),
        tslib_1.__param(7, inversify_1.inject(di_symbols_1.BINDING.Ohno)),
        tslib_1.__param(8, inversify_1.inject(di_symbols_1.BINDING.PRNGFactory)),
        tslib_1.__param(9, inversify_1.inject(di_symbols_1.BINDING.RuntimeData)),
        tslib_1.__param(10, inversify_1.inject(di_symbols_1.BINDING.TaskManager)),
        tslib_1.__param(11, inversify_1.inject(di_symbols_1.BINDING.Util)),
        tslib_1.__param(12, inversify_1.inject(di_symbols_1.BINDING.Config)),
        tslib_1.__metadata("design:paramtypes", [Object, Object, Object, Object, Object, Object, Object, Object, Object, Object, Object, Object, Object])
    ], RuntimeManagerImpl);
    return RuntimeManagerImpl;
}());
exports.RuntimeManagerImpl = RuntimeManagerImpl;
