"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var inversify_1 = require("inversify");
var AssertType;
(function (AssertType) {
    AssertType[AssertType["pass"] = 0] = "pass";
    AssertType[AssertType["fail"] = 1] = "fail";
    AssertType[AssertType["truthy"] = 2] = "truthy";
    AssertType[AssertType["falsy"] = 3] = "falsy";
    AssertType[AssertType["is"] = 4] = "is";
    AssertType[AssertType["not"] = 5] = "not";
})(AssertType = exports.AssertType || (exports.AssertType = {}));
var TestAssertionToolImpl = (function () {
    function TestAssertionToolImpl() {
        this.passed = [];
        this.failed = [];
    }
    TestAssertionToolImpl.prototype.plan = function (n_tests) {
        this.planned = n_tests;
    };
    TestAssertionToolImpl.prototype.fail = function (message) {
        this.failed.push({
            type: AssertType.fail,
            message: message || 'TestBlock[fail]',
        });
    };
    TestAssertionToolImpl.prototype.pass = function (message) {
        this.passed.push({
            type: AssertType.pass,
            message: message || 'TestBlock[pass]',
        });
    };
    TestAssertionToolImpl.prototype.truthy = function (obj, message) {
        var assert = {
            type: AssertType.truthy,
            params: {
                value: obj,
            },
            message: message || '',
        };
        if (!!obj) {
            this.passed.push(assert);
        }
        else {
            assert.message = message || "TestBlock[truthy] Value is not truthy: " + obj;
            this.failed.push(assert);
        }
    };
    TestAssertionToolImpl.prototype.falsy = function (obj, message) {
        var assert = {
            type: AssertType.falsy,
            params: {
                value: obj,
            },
            message: '',
        };
        if (!obj) {
            this.passed.push(assert);
        }
        else {
            assert.message = message || "TestBlock[falsy] Value is not falsy: " + obj;
            this.failed.push(assert);
        }
    };
    TestAssertionToolImpl.prototype.is = function (value, expected, message) {
        var assert = {
            type: AssertType.is,
            params: {
                value: value,
                expected: expected,
            },
            message: message || '',
        };
        if (value === expected) {
            this.passed.push(assert);
        }
        else {
            assert.message = message || "TestBlock[is] Expected [" + expected + "] but got [" + value + "]";
            this.failed.push(assert);
        }
    };
    TestAssertionToolImpl.prototype.not = function (value, expected, message) {
        var assert = {
            type: AssertType.not,
            params: {
                value: value,
                expected: expected,
            },
            message: message || '',
        };
        if (value !== expected) {
            this.passed.push(assert);
        }
        else {
            assert.message = message || "TestBlock[not] Value is the same as: " + value;
            this.failed.push(assert);
        }
    };
    TestAssertionToolImpl.prototype.get_result = function () {
        this.update_planned_result();
        if (this.failed.length > 0) {
            return {
                success: false,
                message: this.failed.map(function (failure) { return failure.message; }).join('\n'),
            };
        }
        if (this.passed.length === 0) {
            return {
                success: false,
                message: "Test finished with no assertion.",
            };
        }
        return {
            success: true,
            message: "Passed " + this.passed.length + " assertion" + this.get_plural_by_amt(this.passed.length) + ".",
        };
    };
    TestAssertionToolImpl.prototype.reset = function () {
        this.failed = [];
        this.passed = [];
        this.planned = undefined;
    };
    TestAssertionToolImpl.prototype.get_plural_by_amt = function (amt) {
        return (amt === 1) ? '' : 's';
    };
    TestAssertionToolImpl.prototype.update_planned_result = function () {
        if (this.planned === undefined) {
            return;
        }
        var num_of_assertion = this.failed.length + this.passed.length;
        this.is(num_of_assertion, this.planned, "Planned for " + this.planned + " assertion" + this.get_plural_by_amt(this.planned) + " but got " + num_of_assertion + ".");
    };
    TestAssertionToolImpl = tslib_1.__decorate([
        inversify_1.injectable()
    ], TestAssertionToolImpl);
    return TestAssertionToolImpl;
}());
exports.TestAssertionToolImpl = TestAssertionToolImpl;
