"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
function get_domain_functions(runtime_manager, runtime_data, event_bus, ohno, config, assertion_tool) {
    var t = assertion_tool;
    var domain_functions = {
        plan: function (args) {
            t.plan(args.n_planned_assertions);
        },
        fail: function (args) {
            t.fail(args.message);
        },
        pass: function (args) {
            t.pass(args.message);
        },
        truthy: function (args) {
            t.truthy(args.value, args.message);
        },
        falsy: function (args) {
            t.falsy(args.value, args.message);
        },
        is: function (args) {
            t.is(args.value, args.expected, args.message);
        },
        not: function (args) {
            t.not(args.value, args.expected, args.message);
        },
        done: function (args) {
            event_bus.runtime_data.test_done.send();
        },
        bad_promise: function (args) {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                return tslib_1.__generator(this, function (_a) {
                    throw ohno.client.domain_function_error(new Error('Threw error in bad_promise on purpose.'), {
                        client_annotation: 'Annotation: Threw error in bad_promise on purpose.',
                    });
                });
            });
        },
        good_promise: function (args) {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                return tslib_1.__generator(this, function (_a) {
                    return [2, new Promise(function (resolve, reject) {
                            setTimeout(function () { return resolve(args.returns); }, args.milliseconds);
                        })];
                });
            });
        },
        call_user_procedure: function (args, interpreter_id, entity_id, internals) {
            internals.add_user_procedure_call_to_stack(args.function_id, entity_id, {
                argone: args.argone,
            });
        },
        send_test_action: function (args, interpreter_id, entity_id, internals) {
            internals.runtime_manager.send_action({
                id: 'testaction',
                namespace: 'AUTOMATEDTESTS',
                parameters: {
                    test_parameter: args.parameter_value,
                },
            });
        },
        get_test_action_parameter: function (args, interpreter_id, entity_id, internals) {
            var param_value = internals.get_action_parameter('test_parameter');
            if (param_value == undefined) {
                throw ohno.system.unknown_system_error({
                    message: 'Could not get parameter in test block "test action parameter"',
                });
            }
            return param_value;
        },
        create_entity_instance: function (args, interpreter_id, entity_id, internals) {
            var typeclass_id = args.typeclass_id;
            var rm = internals.runtime_manager;
            var new_entity_id = rm.generate_random_id('randomid');
            rm.create_entity_instance(typeclass_id, new_entity_id);
        },
        pull_event_test: function (args, interpreter_id, entity_id, internals) {
            return true;
        },
    };
    return function () { return domain_functions; };
}
exports.get_domain_functions = get_domain_functions;
