"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var inversify_1 = require("inversify");
var di_symbols_1 = require("./di_symbols");
var ToolboxImpl = (function () {
    function ToolboxImpl(u, toolbox_config, _block_xml) {
        var _this = this;
        this.u = u;
        this._block_xml = _block_xml;
        this._colors = {};
        this._icons = {};
        this.block_array = [];
        this._id = toolbox_config.id;
        this._type = toolbox_config.type;
        toolbox_config.categories.forEach(function (c) {
            var category_name = c.category_name;
            _this._colors[category_name] = c.color;
            _this._icons[category_name] = c.icon;
            _this.block_array.push(category_name);
            if (c.blocks == undefined) {
                return;
            }
            c.blocks.forEach(function (block_name) { _this.block_array.push(block_name); });
        });
        this._xml = "<xml id=\"blockly-toolbox-xml-" + this._id + "\" style=\"display: none\">";
        toolbox_config.categories.forEach(function (category_config) {
            var custom = category_config.custom || '';
            var $category = "<category name=\"" + category_config.category_name + "\" custom=\"" + custom + "\">";
            if (category_config.blocks != undefined) {
                category_config.blocks.forEach(function (block_name) {
                    var block_xml = _block_xml[block_name];
                    if (!block_xml) {
                        _this.u.log.warn("xml definition of block \"" + block_name + "\" is not found");
                    }
                    $category += block_xml;
                });
            }
            $category += "</category>";
            _this._xml += $category;
        });
        this._xml += "</xml>";
    }
    ToolboxImpl.prototype.get_styles = function () {
        var icons = this._icons;
        var styles = [];
        for (var category_name in this._colors) {
            var color = this._colors[category_name];
            var str = "<style type=\"text/css\">";
            str += "\n          .blocklyToolboxDiv [role=\"treeitem\"][category-name=\"" + category_name + "\"] {\n            border-color: " + color + ";\n          }\n\n          .blocklyToolboxDiv [role=\"treeitem\"][category-name=\"" + category_name + "\"][aria-selected=\"true\"] {\n            background-color: " + color + ";\n          }\n\n          .blocklyToolboxDiv [role=\"treeitem\"][category-name=\"" + category_name + "\"] .blocklyTreeIcon {\n            " + icons[category_name].normal_icon_css + "\n          }\n\n          .blocklyToolboxDiv [role=\"treeitem\"][category-name=\"" + category_name + "\"][aria-selected=\"true\"] .blocklyTreeIcon {\n            " + icons[category_name].selected_icon_css + "\n          }\n        ";
            str += "</style>";
            styles.push(str);
        }
        return styles;
    };
    ToolboxImpl.prototype.set_id = function (id) {
        this._id = id;
    };
    ToolboxImpl.prototype.set_scope = function (scope) {
        this._scope = scope;
    };
    ToolboxImpl.prototype.get_id = function () {
        return this._id;
    };
    ToolboxImpl.prototype.get_xml = function () {
        return this._xml;
    };
    ToolboxImpl.prototype.get_type = function () {
        return this._type;
    };
    ToolboxImpl.prototype.get_block_array = function () {
        return this.block_array;
    };
    ToolboxImpl.prototype.get_scope = function () {
        return this._scope;
    };
    return ToolboxImpl;
}());
var ToolboxFactoryImpl = (function () {
    function ToolboxFactoryImpl(u) {
        this.u = u;
    }
    ToolboxFactoryImpl.prototype.create = function (toolbox_config, block_xml) {
        return new ToolboxImpl(this.u, toolbox_config, block_xml);
    };
    ToolboxFactoryImpl = tslib_1.__decorate([
        inversify_1.injectable(),
        tslib_1.__param(0, inversify_1.inject(di_symbols_1.BINDING.Util)),
        tslib_1.__metadata("design:paramtypes", [Object])
    ], ToolboxFactoryImpl);
    return ToolboxFactoryImpl;
}());
exports.ToolboxFactoryImpl = ToolboxFactoryImpl;
