"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var _cloneDeep = require("lodash/cloneDeep");
var inversify_1 = require("inversify");
var di_symbols_1 = require("./di_symbols");
var UtilImpl = (function () {
    function UtilImpl(config, ohno, log, block) {
        this.config = config;
        this.ohno = ohno;
        this.log = log;
        this.block = block;
        this.misc = {
            ce_restore_cyclical_references: this.ce_restore_cyclical_references.bind(this),
            ce_without_cyclical_references: this.ce_without_cyclical_references.bind(this),
        };
    }
    UtilImpl.prototype.ce_without_cyclical_references = function (ce) {
        var ce_no_ref = _cloneDeep(ce);
        for (var bjs_id in ce_no_ref.compiled_block_map) {
            var cb = ce_no_ref.compiled_block_map[bjs_id];
            this.remove_parent_references(cb);
        }
        for (var p_id in ce_no_ref.procedures) {
            var p_cb = ce_no_ref.procedures[p_id];
            this.remove_parent_references(p_cb);
        }
        return ce_no_ref;
    };
    UtilImpl.prototype.ce_restore_cyclical_references = function (ce) {
        for (var key in ce.compiled_block_map) {
            var b = ce.compiled_block_map[key];
            this.restore_parents(b);
        }
        for (var key in ce.procedures) {
            var b = ce.compiled_block_map[key];
            this.restore_parents(b);
        }
    };
    UtilImpl.prototype.remove_parent_references = function (b) {
        if (!b) {
            return;
        }
        b.parent_block = undefined;
        if (b.next_block) {
            this.remove_parent_references(b.next_block);
        }
        for (var i = 0; i < b.child_block.length; i++) {
            var child = b.child_block[i];
            if (child == undefined) {
                continue;
            }
            this.remove_parent_references(child);
        }
        for (var p_id in b.params) {
            var param = b.params[p_id];
            if (param['parent_block'] != undefined) {
                this.remove_parent_references(param);
            }
        }
        if (this.block.is.cond_block(b)) {
            for (var i = 0; i < b.conditions.length; i++) {
                var condition = b.conditions[i];
                if (condition == undefined) {
                    continue;
                }
                this.remove_parent_references(condition);
            }
        }
    };
    UtilImpl.prototype.restore_parents = function (b, parent_block) {
        if (b == undefined) {
            return;
        }
        b.parent_block = parent_block;
        if (b.next_block) {
            this.restore_parents(b.next_block, parent_block);
        }
        for (var i = 0; i < b.child_block.length; i++) {
            var child = b.child_block[i];
            if (child == undefined) {
                continue;
            }
            this.restore_parents(child, b);
        }
        for (var p_id in b.params) {
            var param = b.params[p_id];
            if (param['parent_block'] != undefined) {
                this.restore_parents(param, b);
            }
        }
        if (this.block.is.cond_block(b)) {
            for (var i = 0; i < b.conditions.length; i++) {
                var condition = b.conditions[i];
                if (condition == undefined) {
                    continue;
                }
                this.restore_parents(condition, b);
            }
        }
    };
    UtilImpl = tslib_1.__decorate([
        inversify_1.injectable(),
        tslib_1.__param(0, inversify_1.inject(di_symbols_1.BINDING.Config)),
        tslib_1.__param(1, inversify_1.inject(di_symbols_1.BINDING.Ohno)),
        tslib_1.__param(2, inversify_1.inject(di_symbols_1.BINDING.Log)),
        tslib_1.__param(3, inversify_1.inject(di_symbols_1.BINDING.BlockUtil)),
        tslib_1.__metadata("design:paramtypes", [Object, Object, Object, Object])
    ], UtilImpl);
    return UtilImpl;
}());
exports.UtilImpl = UtilImpl;
